@@ -36,7 +36,9 @@
 #include "gdata-link.h"
 #include "gdata-parsable.h"
 #include "gdata-parser.h"
+#include "gdata-comparable.h"
 
+static void gdata_link_comparable_init (GDataComparableIface *iface);
 static void gdata_link_finalize (GObject *object);
 static void gdata_link_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_link_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -61,8 +63,8 @@ enum {
 	PROP_LENGTH
 };
 
-G_DEFINE_TYPE (GDataLink, gdata_link, GDATA_TYPE_PARSABLE)
-#define GDATA_LINK_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_LINK, GDataLinkPrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataLink, gdata_link, GDATA_TYPE_PARSABLE,
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_COMPARABLE, gdata_link_comparable_init))
 
 static void
 gdata_link_class_init (GDataLinkClass *klass)
@@ -91,10 +93,10 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_URI,
-				g_param_spec_string ("uri",
-					"URI", "The link's IRI.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("uri",
+	                                                      "URI", "The link's IRI.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataLink:relation-type:
@@ -107,10 +109,10 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_RELATION_TYPE,
-				g_param_spec_string ("relation-type",
-					"Relation type", "The link relation type.",
-					GDATA_LINK_ALTERNATE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("relation-type",
+	                                                      "Relation type", "The link relation type.",
+	                                                      GDATA_LINK_ALTERNATE,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataLink:content-type:
@@ -124,10 +126,10 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
-				g_param_spec_string ("content-type",
-					"Content type", "An advisory media type.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("content-type",
+	                                                      "Content type", "An advisory media type.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataLink:language:
@@ -140,10 +142,10 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LANGUAGE,
-				g_param_spec_string ("language",
-					"Language", "Describes the language of the resource pointed to by the #GDataLink:uri property.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("language",
+	                                                      "Language", "Describes the language of the resource pointed to by the uri property.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataLink:title:
@@ -156,10 +158,10 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TITLE,
-				g_param_spec_string ("title",
-					"Title", "Conveys human-readable information about the link.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("title",
+	                                                      "Title", "Conveys human-readable information about the link.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataLink:length:
@@ -172,10 +174,26 @@ gdata_link_class_init (GDataLinkClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LENGTH,
-				g_param_spec_int ("length",
-					"Length", "Indicates an advisory length of the linked content in octets.",
-					-1, G_MAXINT, -1,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int ("length",
+	                                                   "Length", "Indicates an advisory length of the linked content in octets.",
+	                                                   -1, G_MAXINT, -1,
+	                                                   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
+
+static gint
+compare_with (GDataComparable *self, GDataComparable *other)
+{
+	GDataLinkPrivate *a = ((GDataLink*) self)->priv, *b = ((GDataLink*) other)->priv;
+
+	if (g_strcmp0 (a->uri, b->uri) == 0 && g_strcmp0 (a->relation_type, b->relation_type) == 0)
+		return 0;
+	return 1;
+}
+
+static void
+gdata_link_comparable_init (GDataComparableIface *iface)
+{
+	iface->compare_with = compare_with;
 }
 
 static void
@@ -355,37 +373,6 @@ gdata_link_new (const gchar *uri, const gchar *relation_type)
 	return g_object_new (GDATA_TYPE_LINK, "uri", uri, "relation-type", relation_type, NULL);
 }
 
-/**
- * gdata_link_compare:
- * @a: a #GDataLink, or %NULL
- * @b: another #GDataLink, or %NULL
- *
- * Compares the two links in a strcmp() fashion. %NULL values are handled gracefully, with
- * <code class="literal">0</code> returned if both @a and @b are %NULL, <code class="literal">-1</code> if @a is %NULL
- * and <code class="literal">1</code> if @b is %NULL.
- *
- * The comparison of non-%NULL values is done on the basis of the @uri property of the #GDataLink<!-- -->s.
- *
- * Return value: <code class="literal">0</code> if @a equals @b, <code class="literal">-1</code> or <code class="literal">1</code> as
- * appropriate otherwise
- *
- * Since: 0.4.0
- **/
-gint
-gdata_link_compare (const GDataLink *a, const GDataLink *b)
-{
-	if (a == NULL && b != NULL)
-		return -1;
-	else if (a != NULL && b == NULL)
-		return 1;
-
-	if (a == b)
-		return 0;
-	if (g_strcmp0 (a->priv->uri, b->priv->uri) == 0 && g_strcmp0 (a->priv->relation_type, b->priv->relation_type) == 0)
-		return 0;
-	return 1;
-}
-
 /**
  * gdata_link_get_uri:
  * @self: a #GDataLink
