@@ -45,6 +45,7 @@
 #include "gdata-private.h"
 
 static void gdata_download_stream_seekable_iface_init (GSeekableIface *seekable_iface);
+static GObject *gdata_download_stream_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params);
 static void gdata_download_stream_dispose (GObject *object);
 static void gdata_download_stream_finalize (GObject *object);
 static void gdata_download_stream_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
@@ -85,8 +86,7 @@ enum {
 };
 
 G_DEFINE_TYPE_WITH_CODE (GDataDownloadStream, gdata_download_stream, G_TYPE_INPUT_STREAM,
-			 G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE, gdata_download_stream_seekable_iface_init))
-#define GDATA_DOWNLOAD_STREAM_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_DOWNLOAD_STREAM, GDataDownloadStreamPrivate))
+                         G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE, gdata_download_stream_seekable_iface_init))
 
 static void
 gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
@@ -96,6 +96,7 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 
 	g_type_class_add_private (klass, sizeof (GDataDownloadStreamPrivate));
 
+	gobject_class->constructor = gdata_download_stream_constructor;
 	gobject_class->dispose = gdata_download_stream_dispose;
 	gobject_class->finalize = gdata_download_stream_finalize;
 	gobject_class->get_property = gdata_download_stream_get_property;
@@ -114,10 +115,10 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SERVICE,
-					 g_param_spec_object ("service",
-							      "Service", "The service which is used to authenticate the download.",
-							      GDATA_TYPE_SERVICE,
-							      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_object ("service",
+	                                                      "Service", "The service which is used to authenticate the download.",
+	                                                      GDATA_TYPE_SERVICE,
+	                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDownloadStream:download-uri:
@@ -127,10 +128,10 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_DOWNLOAD_URI,
-					 g_param_spec_string ("download-uri",
-							      "Download URI", "The URI of the file to download.",
-							      NULL,
-							      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("download-uri",
+	                                                      "Download URI", "The URI of the file to download.",
+	                                                      NULL,
+	                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDownloadStream:content-type:
@@ -140,10 +141,10 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CONTENT_TYPE,
-					 g_param_spec_string ("content-type",
-							      "Content type", "The content type of the file being downloaded.",
-							      NULL,
-							      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("content-type",
+	                                                      "Content type", "The content type of the file being downloaded.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDownloadStream:content-length:
@@ -153,10 +154,10 @@ gdata_download_stream_class_init (GDataDownloadStreamClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CONTENT_LENGTH,
-					 g_param_spec_long ("content-length",
-							    "Content length", "The length (in bytes) of the file being downloaded.",
-							    -1, G_MAXSSIZE, -1,
-							    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_long ("content-length",
+	                                                    "Content length", "The length (in bytes) of the file being downloaded.",
+	                                                    -1, G_MAXSSIZE, -1,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -178,10 +179,37 @@ gdata_download_stream_init (GDataDownloadStream *self)
 	g_static_mutex_init (&(self->priv->content_mutex));
 }
 
+static GObject *
+gdata_download_stream_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params)
+{
+	GDataDownloadStreamPrivate *priv;
+	GDataServiceClass *klass;
+	GObject *object;
+
+	/* Chain up to the parent class */
+	object = G_OBJECT_CLASS (gdata_download_stream_parent_class)->constructor (type, n_construct_params, construct_params);
+	priv = GDATA_DOWNLOAD_STREAM (object)->priv;
+
+	/* Build the message */
+	priv->message = soup_message_new (SOUP_METHOD_GET, priv->download_uri);
+
+	/* Make sure the headers are set */
+	klass = GDATA_SERVICE_GET_CLASS (priv->service);
+	if (klass->append_query_headers != NULL)
+		klass->append_query_headers (priv->service, priv->message);
+
+	/* We don't want to accumulate chunks */
+	soup_message_body_set_accumulate (priv->message->request_body, FALSE);
+
+	/* Downloading doesn't actually start until the first call to read() */
+
+	return object;
+}
+
 static void
 gdata_download_stream_dispose (GObject *object)
 {
-	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM_GET_PRIVATE (object);
+	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM (object)->priv;
 
 	if (priv->service != NULL)
 		g_object_unref (priv->service);
@@ -198,7 +226,7 @@ gdata_download_stream_dispose (GObject *object)
 static void
 gdata_download_stream_finalize (GObject *object)
 {
-	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM_GET_PRIVATE (object);
+	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM (object)->priv;
 
 	g_thread_join (priv->network_thread);
 	g_static_mutex_free (&(priv->content_mutex));
@@ -213,7 +241,7 @@ gdata_download_stream_finalize (GObject *object)
 static void
 gdata_download_stream_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM_GET_PRIVATE (object);
+	GDataDownloadStreamPrivate *priv = GDATA_DOWNLOAD_STREAM (object)->priv;
 
 	switch (property_id) {
 		case PROP_SERVICE:
@@ -466,30 +494,10 @@ create_network_thread (GDataDownloadStream *self, GError **error)
 GInputStream *
 gdata_download_stream_new (GDataService *service, const gchar *download_uri)
 {
-	GDataServiceClass *klass;
-	GDataDownloadStream *download_stream;
-	SoupMessage *message;
-
 	g_return_val_if_fail (GDATA_IS_SERVICE (service), NULL);
 	g_return_val_if_fail (download_uri != NULL, NULL);
 
-	/* Build the message */
-	message = soup_message_new (SOUP_METHOD_GET, download_uri);
-
-	/* Make sure the headers are set */
-	klass = GDATA_SERVICE_GET_CLASS (service);
-	if (klass->append_query_headers != NULL)
-		klass->append_query_headers (GDATA_SERVICE (service), message);
-
-	/* We don't want to accumulate chunks */
-	soup_message_body_set_accumulate (message->request_body, FALSE);
-
-	download_stream = g_object_new (GDATA_TYPE_DOWNLOAD_STREAM, "download-uri", download_uri, "service", service, NULL);
-	download_stream->priv->message = message;
-
-	/* Downloading doesn't actually start until the first call to read() */
-
-	return G_INPUT_STREAM (download_stream);
+	return G_INPUT_STREAM (g_object_new (GDATA_TYPE_DOWNLOAD_STREAM, "download-uri", download_uri, "service", service, NULL));
 }
 
 /**
@@ -590,7 +598,8 @@ gdata_download_stream_get_content_length (GDataDownloadStream *self)
  * Since: 0.6.0
  **/
 GFileOutputStream *
-_gdata_download_stream_find_destination (const gchar *default_filename, GFile *target_dest_file, GFile **actual_dest_file, gboolean replace_file_if_exists, GCancellable *cancellable, GError **error)
+_gdata_download_stream_find_destination (const gchar *default_filename, GFile *target_dest_file, GFile **actual_dest_file,
+                                         gboolean replace_file_if_exists, GCancellable *cancellable, GError **error)
 {
 	GFileInfo *target_dest_info;
 	GFileOutputStream *dest_stream;
