@@ -36,6 +36,8 @@ G_BEGIN_DECLS
  * @GDATA_OPERATION_DELETION: a deletion of a #GDataEntry
  * @GDATA_OPERATION_DOWNLOAD: a download of a file
  * @GDATA_OPERATION_UPLOAD: an upload of a file
+ * @GDATA_OPERATION_AUTHENTICATION: authentication with the service
+ * @GDATA_OPERATION_BATCH: a batch operation with #GDataBatchOperation
  *
  * Representations of the different operations performed by the library.
  *
@@ -47,12 +49,14 @@ typedef enum {
 	GDATA_OPERATION_UPDATE,
 	GDATA_OPERATION_DELETION,
 	GDATA_OPERATION_DOWNLOAD,
-	GDATA_OPERATION_UPLOAD
+	GDATA_OPERATION_UPLOAD,
+	GDATA_OPERATION_AUTHENTICATION,
+	GDATA_OPERATION_BATCH
 } GDataOperationType;
 
 /**
  * GDataServiceError:
- * @GDATA_SERVICE_ERROR_UNAVAILABLE: The service is unavailable due to maintainence or other reasons
+ * @GDATA_SERVICE_ERROR_UNAVAILABLE: The service is unavailable due to maintainence or other reasons (e.g. network errors at the server end)
  * @GDATA_SERVICE_ERROR_PROTOCOL_ERROR: The client or server unexpectedly strayed from the protocol (fatal error)
  * @GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED: An entry has already been inserted, and cannot be re-inserted
  * @GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED: The user attempted to do something which required authentication, and they weren't authenticated
@@ -61,6 +65,9 @@ typedef enum {
  * and uploading the modified entry
  * @GDATA_SERVICE_ERROR_FORBIDDEN: Generic error for a forbidden action (not due to having insufficient permissions)
  * @GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER: A given query parameter was invalid for the query type
+ * @GDATA_SERVICE_ERROR_NETWORK_ERROR: The service is unavailable due to local network errors (e.g. no Internet connection)
+ * @GDATA_SERVICE_ERROR_PROXY_ERROR: The service is unavailable due to proxy network errors (e.g. proxy unreachable)
+ * @GDATA_SERVICE_ERROR_WITH_BATCH_OPERATION: Generic error when running a batch operation and the whole operation fails
  *
  * Error codes for #GDataService operations.
  **/
@@ -72,7 +79,10 @@ typedef enum {
 	GDATA_SERVICE_ERROR_NOT_FOUND,
 	GDATA_SERVICE_ERROR_CONFLICT,
 	GDATA_SERVICE_ERROR_FORBIDDEN,
-	GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER
+	GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER,
+	GDATA_SERVICE_ERROR_NETWORK_ERROR,
+	GDATA_SERVICE_ERROR_PROXY_ERROR,
+	GDATA_SERVICE_ERROR_WITH_BATCH_OPERATION
 } GDataServiceError;
 
 /**
@@ -82,14 +92,17 @@ typedef enum {
  * directly to resolve the issue before logging in using a non-Google application.
  * @GDATA_AUTHENTICATION_ERROR_TERMS_NOT_AGREED: The user has not agreed to terms. The user will need to access their Google account directly to
  * resolve the issue before logging in using a non-Google application.
- * @GDATA_AUTHENTICATION_ERROR_CAPTCHA_REQUIRED: A CAPTCHA is required. (A response with this error code will also contain an image URL and a
+ * @GDATA_AUTHENTICATION_ERROR_CAPTCHA_REQUIRED: A CAPTCHA is required. (A response with this error code will also contain an image URI and a
  * CAPTCHA token.)
  * @GDATA_AUTHENTICATION_ERROR_ACCOUNT_DELETED: The user account has been deleted.
  * @GDATA_AUTHENTICATION_ERROR_ACCOUNT_DISABLED: The user account has been disabled.
  * @GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED: The user's access to the specified service has been disabled. (The user account may still be valid.)
+ * @GDATA_AUTHENTICATION_ERROR_ACCOUNT_MIGRATED: The user's account login details have been migrated to a new system. (This is used for the transition
+ * from the old YouTube login details to the new ones.)
  *
- * Error codes for #GDataDataService authentication operations. See http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html#Errors for
- * the official reference.
+ * Error codes for #GDataService authentication operations. See the
+ * <ulink type="http" url="http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html#Errors">Google accounts documentation</ulink> for
+ * more information.
  **/
 typedef enum {
 	GDATA_AUTHENTICATION_ERROR_BAD_AUTHENTICATION = 1,
@@ -98,7 +111,8 @@ typedef enum {
 	GDATA_AUTHENTICATION_ERROR_CAPTCHA_REQUIRED,
 	GDATA_AUTHENTICATION_ERROR_ACCOUNT_DELETED,
 	GDATA_AUTHENTICATION_ERROR_ACCOUNT_DISABLED,
-	GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED
+	GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED,
+	GDATA_AUTHENTICATION_ERROR_ACCOUNT_MIGRATED
 } GDataAuthenticationError;
 
 /**
@@ -143,7 +157,7 @@ typedef struct {
  * @parent: the parent class
  * @service_name: the name of the service (for subclasses) as given in the service's GData API documentation
  * @authentication_uri: the authentication URI (for subclasses) if different from the Google ClientLogin default
- * @api_version: the version of the GData API used by the service (typically %2)
+ * @api_version: the version of the GData API used by the service (typically <code class="literal">2</code>)
  * @feed_type: the #GType of the feed class (subclass of #GDataFeed) to use for query results from this service
  * @parse_authentication_response: a function to parse the response from the online service to an authentication request as
  * issued by gdata_service_authenticate(). It should return %TRUE if authentication was successful, and %FALSE if there was
@@ -182,36 +196,53 @@ gboolean gdata_service_authenticate_finish (GDataService *self, GAsyncResult *as
 
 GDataFeed *gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
                                 GCancellable *cancellable,
-                                GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+                                GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
                                 GCancellable *cancellable,
                                 GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
                                 GAsyncReadyCallback callback, gpointer user_data);
-GDataFeed *gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+GDataFeed *gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+
+GDataEntry *gdata_service_query_single_entry (GDataService *self, const gchar *entry_id, GDataQuery *query, GType entry_type,
+                                              GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void gdata_service_query_single_entry_async (GDataService *self, const gchar *entry_id, GDataQuery *query, GType entry_type,
+                                             GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data);
+GDataEntry *gdata_service_query_single_entry_finish (GDataService *self, GAsyncResult *async_result,
+                                                     GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
 GDataEntry *gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEntry *entry,
-                                        GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+                                        GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_service_insert_entry_async (GDataService *self, const gchar *upload_uri, GDataEntry *entry, GCancellable *cancellable,
                                        GAsyncReadyCallback callback, gpointer user_data);
-GDataEntry *gdata_service_insert_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+GDataEntry *gdata_service_insert_entry_finish (GDataService *self, GAsyncResult *async_result,
+                                               GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
-GDataEntry *gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+GDataEntry *gdata_service_update_entry (GDataService *self, GDataEntry *entry,
+                                        GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_service_update_entry_async (GDataService *self, GDataEntry *entry, GCancellable *cancellable,
                                        GAsyncReadyCallback callback, gpointer user_data);
-GDataEntry *gdata_service_update_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+GDataEntry *gdata_service_update_entry_finish (GDataService *self, GAsyncResult *async_result,
+                                               GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
 gboolean gdata_service_delete_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error);
 void gdata_service_delete_entry_async (GDataService *self, GDataEntry *entry, GCancellable *cancellable,
                                        GAsyncReadyCallback callback, gpointer user_data);
 gboolean gdata_service_delete_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error);
 
-SoupURI *gdata_service_get_proxy_uri (GDataService *self);
+SoupURI *gdata_service_get_proxy_uri (GDataService *self) G_GNUC_PURE;
 void gdata_service_set_proxy_uri (GDataService *self, SoupURI *proxy_uri);
 
-gboolean gdata_service_is_authenticated (GDataService *self);
-const gchar *gdata_service_get_client_id (GDataService *self);
-const gchar *gdata_service_get_username (GDataService *self);
-const gchar *gdata_service_get_password (GDataService *self);
+guint gdata_service_get_timeout (GDataService *self) G_GNUC_PURE;
+void gdata_service_set_timeout (GDataService *self, guint timeout);
+
+gboolean gdata_service_is_authenticated (GDataService *self) G_GNUC_PURE;
+const gchar *gdata_service_get_client_id (GDataService *self) G_GNUC_PURE;
+const gchar *gdata_service_get_username (GDataService *self) G_GNUC_PURE;
+const gchar *gdata_service_get_password (GDataService *self) G_GNUC_PURE;
+
+const gchar *gdata_service_get_locale (GDataService *self) G_GNUC_PURE;
+void gdata_service_set_locale (GDataService *self, const gchar *locale);
 
 G_END_DECLS
 
