@@ -26,6 +26,8 @@
  * #GDataYouTubeCredit represents the YouTube-specific customizations to #GDataMediaCredit. For more information,
  * see the <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_media:credit">
  * online documentation</ulink>.
+ *
+ * Since: 0.4.0
  **/
 
 #include <glib.h>
@@ -50,7 +52,6 @@ enum {
 };
 
 G_DEFINE_TYPE (GDataYouTubeCredit, gdata_youtube_credit, GDATA_TYPE_MEDIA_CREDIT)
-#define GDATA_YOUTUBE_CREDIT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_YOUTUBE_CREDIT, GDataYouTubeCreditPrivate))
 
 static void
 gdata_youtube_credit_class_init (GDataYouTubeCreditClass *klass)
@@ -70,7 +71,7 @@ gdata_youtube_credit_class_init (GDataYouTubeCreditClass *klass)
 	/**
 	 * GDataYouTubeCredit:entity-type:
 	 *
-	 * The type of entity who is credited. Currently this can only be "partner", for a YouTube partner.
+	 * The type of entity who is credited. Currently this can only be %GDATA_YOUTUBE_CREDIT_ENTITY_PARTNER, for a YouTube partner.
 	 *
 	 * For more information, see the
 	 * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_media:credit">
@@ -79,10 +80,10 @@ gdata_youtube_credit_class_init (GDataYouTubeCreditClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ENTITY_TYPE,
-				g_param_spec_string ("entity-type",
-					"Entity type", "The type of entity who is credited.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("entity-type",
+	                                                      "Entity type", "The type of entity who is credited.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -156,7 +157,7 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
  *
  * Gets the #GDataYouTubeCredit:entity-type property.
  *
- * Return value: the type of the credited user (e.g. "partner"), or %NULL
+ * Return value: the type of the credited user (e.g. %GDATA_YOUTUBE_CREDIT_ENTITY_PARTNER), or %NULL
  *
  * Since: 0.4.0
  **/
