@@ -28,6 +28,36 @@
  *
  * For more details of YouTube's GData API, see the <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html">
  * online documentation</ulink>.
+ *
+ * <example>
+ * 	<title>Getting a Localized List of YouTube Categories</title>
+ * 	<programlisting>
+ *	GDataYouTubeService *service;
+ *	GDataAPPCategories *app_categories;
+ *	GList *categories, *i;
+ *
+ *	/<!-- -->* Create a service and set its locale to Italian, which localizes the categories to Italian *<!-- -->/
+ *	service = create_youtube_service ();
+ *	gdata_service_set_locale (GDATA_SERVICE (service), "it");
+ *
+ *	/<!-- -->* Query the server for the current list of YouTube categories (in Italian) *<!-- -->/
+ *	app_categories = gdata_youtube_service_get_categories (service, NULL, NULL);
+ *	categories = gdata_app_categories_get_categories (app_categories);
+ *
+ *	/<!-- -->* Iterate through the categories *<!-- -->/
+ *	for (i = categories; i != NULL; i = i->next) {
+ *		GDataYouTubeCategory *category = GDATA_YOUTUBE_CATEGORY (i->data);
+ *
+ *		if (gdata_youtube_category_is_deprecated (category) == FALSE && gdata_youtube_category_is_browsable (category, "IT") == TRUE) {
+ *			/<!-- -->* Do something with the category here, as it's not deprecated, and is browsable in the given region *<!-- -->/
+ *			add_to_ui (gdata_category_get_term (GDATA_CATEGORY (category)), gdata_category_get_label (GDATA_CATEGORY (category)));
+ *		}
+ *	}
+ *
+ *	g_object_unref (app_categories);
+ *	g_object_unref (service);
+ * 	</programlisting>
+ * </example>
  **/
 
 #include <config.h>
@@ -42,6 +72,8 @@
 #include "gdata-parser.h"
 #include "atom/gdata-link.h"
 #include "gdata-upload-stream.h"
+#include "gdata-youtube-category.h"
+#include "gdata-batchable.h"
 
 /* Standards reference here: http://code.google.com/apis/youtube/2.0/reference.html */
 
@@ -69,8 +101,7 @@ enum {
 	PROP_YOUTUBE_USER
 };
 
-G_DEFINE_TYPE (GDataYouTubeService, gdata_youtube_service, GDATA_TYPE_SERVICE)
-#define GDATA_YOUTUBE_SERVICE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_YOUTUBE_SERVICE, GDataYouTubeServicePrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataYouTubeService, gdata_youtube_service, GDATA_TYPE_SERVICE, G_IMPLEMENT_INTERFACE (GDATA_TYPE_BATCHABLE, NULL))
 
 static void
 gdata_youtube_service_class_init (GDataYouTubeServiceClass *klass)
@@ -99,10 +130,10 @@ gdata_youtube_service_class_init (GDataYouTubeServiceClass *klass)
 	 * The matching #GDataService:client-id property belongs to #GDataService.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_DEVELOPER_KEY,
-				g_param_spec_string ("developer-key",
-					"Developer key", "Your YouTube developer API key.",
-					NULL,
-					G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("developer-key",
+	                                                      "Developer key", "Your YouTube developer API key.",
+	                                                      NULL,
+	                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataYouTubeService:youtube-user:
@@ -111,10 +142,10 @@ gdata_youtube_service_class_init (GDataYouTubeServiceClass *klass)
 	 * YouTube was converted to use Google's centralised login system.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_YOUTUBE_USER,
-				g_param_spec_string ("youtube-user",
-					"YouTube username", "The YouTube account username.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("youtube-user",
+	                                                      "YouTube username", "The YouTube account username.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -126,7 +157,7 @@ gdata_youtube_service_init (GDataYouTubeService *self)
 static void
 gdata_youtube_service_finalize (GObject *object)
 {
-	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE_GET_PRIVATE (object);
+	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (object)->priv;
 
 	g_free (priv->youtube_user);
 	g_free (priv->developer_key);
@@ -138,7 +169,7 @@ gdata_youtube_service_finalize (GObject *object)
 static void
 gdata_youtube_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE_GET_PRIVATE (object);
+	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (object)->priv;
 
 	switch (property_id) {
 		case PROP_DEVELOPER_KEY:
@@ -157,7 +188,7 @@ gdata_youtube_service_get_property (GObject *object, guint property_id, GValue *
 static void
 gdata_youtube_service_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
-	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE_GET_PRIVATE (object);
+	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (object)->priv;
 
 	switch (property_id) {
 		case PROP_DEVELOPER_KEY:
@@ -178,7 +209,7 @@ parse_authentication_response (GDataService *self, guint status, const gchar *re
 
 	/* Chain up to the parent method first */
 	if (GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->parse_authentication_response (self, status,
-												     response_body, length, error) == FALSE) {
+		                                                                                     response_body, length, error) == FALSE) {
 		return FALSE;
 	}
 
@@ -200,7 +231,7 @@ parse_authentication_response (GDataService *self, guint status, const gchar *re
 
 protocol_error:
 	g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-			     _("The server returned a malformed response."));
+	                     _("The server returned a malformed response."));
 	return FALSE;
 }
 
@@ -217,8 +248,6 @@ append_query_headers (GDataService *self, SoupMessage *message)
 	soup_message_headers_append (message->request_headers, "X-GData-Key", key_header);
 	g_free (key_header);
 
-	soup_message_headers_append (message->request_headers, "X-GData-Client", gdata_service_get_client_id (self));
-
 	/* Chain up to the parent class */
 	GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->append_query_headers (self, message);
 }
@@ -296,39 +325,39 @@ parse_error_response (GDataService *self, GDataOperationType operation_type, gui
 		}
 
 		/* Create an error message, but only for the first error */
-		if (error == NULL || *error == NULL) {
+		if (*error == NULL) {
 			/* See http://code.google.com/apis/youtube/2.0/developers_guide_protocol.html#Error_responses */
 			if (xmlStrcmp (domain, (xmlChar*) "yt:service") == 0 && xmlStrcmp (code, (xmlChar*) "disabled_in_maintenance_mode") == 0) {
 				/* Service disabled */
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_UNAVAILABLE,
-					     _("This service is not available at the moment."));
+				             _("This service is not available at the moment."));
 			} else if (xmlStrcmp (domain, (xmlChar*) "yt:authentication") == 0) {
 				/* Authentication problem; make sure to set our status as unauthenticated */
 				_gdata_service_set_authenticated (GDATA_SERVICE (self), FALSE);
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-					     _("You must be authenticated to do this."));
+				             _("You must be authenticated to do this."));
 			} else if (xmlStrcmp (domain, (xmlChar*) "yt:quota") == 0) {
 				/* Quota errors */
 				if (xmlStrcmp (code, (xmlChar*) "too_many_recent_calls") == 0) {
 					g_set_error (error, GDATA_YOUTUBE_SERVICE_ERROR, GDATA_YOUTUBE_SERVICE_ERROR_API_QUOTA_EXCEEDED,
-						     _("You have made too many API calls recently. Please wait a few minutes and try again."));
+					             _("You have made too many API calls recently. Please wait a few minutes and try again."));
 				} else if (xmlStrcmp (code, (xmlChar*) "too_many_entries") == 0) {
 					g_set_error (error, GDATA_YOUTUBE_SERVICE_ERROR, GDATA_YOUTUBE_SERVICE_ERROR_ENTRY_QUOTA_EXCEEDED,
-						     _("You have exceeded your entry quota. Please delete some entries and try again."));
+					             _("You have exceeded your entry quota. Please delete some entries and try again."));
 				} else {
 					/* Protocol error */
 					g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-						     /* Translators: the first parameter is an error code, which is a coded string. The second parameter
-						      * is an error domain, which is another coded string. The third parameter is the location of the
-						      * error, which is either a URI or an XPath. */
-						     _("Unknown error code \"%s\" in domain \"%s\" received with location \"%s\"."),
-						     code, domain, location);
+					             /* Translators: the first parameter is an error code, which is a coded string.
+					              * The second parameter is an error domain, which is another coded string.
+					              * The third parameter is the location of the error, which is either a URI or an XPath. */
+					             _("Unknown error code \"%s\" in domain \"%s\" received with location \"%s\"."),
+					             code, domain, location);
 				}
 			} else {
 				/* Unknown or validation (protocol) error */
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-					     _("Unknown error code \"%s\" in domain \"%s\" received with location \"%s\"."),
-					     code, domain, location);
+				             _("Unknown error code \"%s\" in domain \"%s\" received with location \"%s\"."),
+				             code, domain, location);
 			}
 		} else {
 			/* For all errors after the first, log the error in the terminal */
@@ -344,7 +373,7 @@ parse_error_response (GDataService *self, GDataOperationType operation_type, gui
 
 check_error:
 	/* Ensure we're actually set an error message */
-	if (error != NULL && *error == NULL)
+	if (*error == NULL)
 		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR, _("Unknown and unparsable error received."));
 
 	return;
@@ -372,9 +401,9 @@ gdata_youtube_service_new (const gchar *developer_key, const gchar *client_id)
 	g_return_val_if_fail (client_id != NULL, NULL);
 
 	return g_object_new (GDATA_TYPE_YOUTUBE_SERVICE,
-			     "developer-key", developer_key,
-			     "client-id", client_id,
-			     NULL);
+	                     "developer-key", developer_key,
+	                     "client-id", client_id,
+	                     NULL);
 }
 
 static const gchar *
@@ -424,12 +453,17 @@ standard_feed_type_to_feed_uri (GDataYouTubeStandardFeedType feed_type)
  **/
 GDataFeed *
 gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTubeStandardFeedType feed_type, GDataQuery *query,
-					   GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					   GError **error)
+                                           GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                           GError **error)
 {
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
 	/* TODO: Support the "time" parameter, as well as category- and region-specific feeds */
 	return gdata_service_query (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
-				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
+	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -453,11 +487,16 @@ gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTu
  **/
 void
 gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDataYouTubeStandardFeedType feed_type, GDataQuery *query,
-						 GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						 GAsyncReadyCallback callback, gpointer user_data)
+                                                 GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                 GAsyncReadyCallback callback, gpointer user_data)
 {
+	g_return_if_fail (GDATA_IS_YOUTUBE_SERVICE (self));
+	g_return_if_fail (query == NULL || GDATA_IS_QUERY (query));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
+
 	gdata_service_query_async (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
-				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
+	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -478,11 +517,16 @@ gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDat
  **/
 GDataFeed *
 gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query,
-				    GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-				    GError **error)
+                                    GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                    GError **error)
 {
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
 	return gdata_service_query (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
-				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
+	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -505,170 +549,16 @@ gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query
  **/
 void
 gdata_youtube_service_query_videos_async (GDataYouTubeService *self, GDataQuery *query,
-					  GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					  GAsyncReadyCallback callback, gpointer user_data)
+                                          GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                          GAsyncReadyCallback callback, gpointer user_data)
 {
-	gdata_service_query_async (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
-				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
-}
-
-/**
- * gdata_youtube_service_query_single_video:
- * @self: a #GDataYouTubeService
- * @query: a #GDataQuery with the query parameters, or %NULL
- * @video_id: the video ID of the desired video
- * @cancellable: a #GCancellable, or %NULL
- * @error: a #GError, or %NULL
- *
- * Retrieves information about the single video with the given @video_id. @video_id should be as returned by
- * gdata_youtube_video_get_video_id() or as extracted from a YouTube URI.
- *
- * Parameters and errors are as for gdata_service_query().
- *
- * Return value: (transfer full): a #GDataYouTubeVideo, or %NULL; unref with g_object_unref()
- *
- * Since: 0.4.0
- **/
-GDataYouTubeVideo *
-gdata_youtube_service_query_single_video (GDataYouTubeService *self, GDataQuery *query, const gchar *video_id,
-					  GCancellable *cancellable, GError **error)
-{
-	gchar *feed_uri;
-	GDataParsable *video;
-	SoupMessage *message;
-
-	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
-	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
-	g_return_val_if_fail (video_id != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-
-	/* Query for just the specified video ID */
-	feed_uri = g_strconcat ("http://gdata.youtube.com/feeds/api/videos/", video_id, NULL);
-	message = _gdata_service_query (GDATA_SERVICE (self), feed_uri, query, cancellable, NULL, NULL, error);
-	g_free (feed_uri);
-
-	if (message == NULL)
-		return NULL;
-
-	g_assert (message->response_body->data != NULL);
-	video = gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO, message->response_body->data, message->response_body->length, error);
-	g_object_unref (message);
-
-	return GDATA_YOUTUBE_VIDEO (video);
-}
-
-typedef struct {
-	GDataQuery *query;
-	gchar *video_id;
-} QuerySingleVideoAsyncData;
-
-static void
-query_single_video_async_data_free (QuerySingleVideoAsyncData *data)
-{
-	g_free (data->video_id);
-	if (data->query != NULL)
-		g_object_unref (data->query);
-	g_slice_free (QuerySingleVideoAsyncData, data);
-}
-
-static void
-query_single_video_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *cancellable)
-{
-	GDataYouTubeVideo *video;
-	GError *error = NULL;
-	QuerySingleVideoAsyncData *data = g_simple_async_result_get_op_res_gpointer (result);
-
-	/* Check to see if it's been cancelled already */
-	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
-		g_simple_async_result_set_from_error (result, error);
-		g_error_free (error);
-		return;
-	}
-
-	/* Execute the query and return */
-	video = gdata_youtube_service_query_single_video (GDATA_YOUTUBE_SERVICE (service), data->query, data->video_id,
-							  cancellable, &error);
-	if (video == NULL) {
-		g_simple_async_result_set_from_error (result, error);
-		g_error_free (error);
-	}
-
-	g_simple_async_result_set_op_res_gpointer (result, video, (GDestroyNotify) g_object_unref);
-}
-
-/**
- * gdata_youtube_service_query_single_video_async:
- * @self: a #GDataService
- * @query: a #GDataQuery with the query parameters, or %NULL
- * @video_id: the video ID of the desired video
- * @cancellable: optional #GCancellable object, or %NULL
- * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: data to pass to the @callback function
- *
- * Retrieves information about the single video with the given @video_id. @video_id should be as returned by
- * gdata_youtube_video_get_video_id() or as extracted from a YouTube URI. @self, @query and @video_id are reffed/copied when this
- * function is called, so can safely be freed after this function returns.
- *
- * For more details, see gdata_youtube_service_query_single_video(), which is the synchronous version of this function.
- *
- * When the operation is finished, @callback will be called. You can then call gdata_youtube_service_query_single_video_finish()
- * to get the results of the operation.
- *
- * Since: 0.4.0
- **/
-void
-gdata_youtube_service_query_single_video_async (GDataYouTubeService *self, GDataQuery *query, const gchar *video_id,
-						GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
-{
-	GSimpleAsyncResult *result;
-	QuerySingleVideoAsyncData *data;
-
 	g_return_if_fail (GDATA_IS_YOUTUBE_SERVICE (self));
 	g_return_if_fail (query == NULL || GDATA_IS_QUERY (query));
-	g_return_if_fail (video_id != NULL);
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
 
-	data = g_slice_new (QuerySingleVideoAsyncData);
-	data->query = (query != NULL) ? g_object_ref (query) : NULL;
-	data->video_id = g_strdup (video_id);
-
-	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_youtube_service_query_single_video_async);
-	g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify) query_single_video_async_data_free);
-	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) query_single_video_thread, G_PRIORITY_DEFAULT, cancellable);
-	g_object_unref (result);
-}
-
-/**
- * gdata_youtube_service_query_single_video_finish:
- * @self: a #GDataYouTubeService
- * @async_result: a #GAsyncResult
- * @error: a #GError, or %NULL
- *
- * Finishes an asynchronous query operation for a single video, as started with gdata_youtube_service_query_single_video_async().
- *
- * Return value: (transfer full): a #GDataYouTubeVideo, or %NULL; unref with g_object_unref()
- *
- * Since: 0.4.0
- **/
-GDataYouTubeVideo *
-gdata_youtube_service_query_single_video_finish (GDataYouTubeService *self, GAsyncResult *async_result, GError **error)
-{
-	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
-	GDataYouTubeVideo *video;
-
-	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), NULL);
-
-	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == gdata_youtube_service_query_single_video_async);
-
-	if (g_simple_async_result_propagate_error (result, error) == TRUE)
-		return NULL;
-
-	video = g_simple_async_result_get_op_res_gpointer (result);
-	if (video != NULL)
-		return g_object_ref (video);
-
-	g_assert_not_reached ();
+	gdata_service_query_async (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
+	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -690,23 +580,29 @@ gdata_youtube_service_query_single_video_finish (GDataYouTubeService *self, GAsy
  **/
 GDataFeed *
 gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVideo *video, GDataQuery *query,
-				     GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-				     GError **error)
+                                     GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                     GError **error)
 {
 	GDataLink *related_link;
 
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (video), NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
 	/* See if the video already has a rel="http://gdata.youtube.com/schemas/2007#video.related" link */
 	related_link = gdata_entry_look_up_link (GDATA_ENTRY (video), "http://gdata.youtube.com/schemas/2007#video.related");
 	if (related_link == NULL) {
 		/* Erroring out is probably the safest thing to do */
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-				     _("The video did not have a related videos <link>."));
+		                     _("The video did not have a related videos <link>."));
 		return NULL;
 	}
 
 	/* Execute the query */
 	return gdata_service_query (GDATA_SERVICE (self), gdata_link_get_uri (related_link), query,
-				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
+	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -730,23 +626,29 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
  **/
 void
 gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTubeVideo *video, GDataQuery *query,
-					   GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					   GAsyncReadyCallback callback, gpointer user_data)
+                                           GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                           GAsyncReadyCallback callback, gpointer user_data)
 {
 	GDataLink *related_link;
 
+	g_return_if_fail (GDATA_IS_YOUTUBE_SERVICE (self));
+	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (video));
+	g_return_if_fail (query == NULL || GDATA_IS_QUERY (query));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
+
 	/* See if the video already has a rel="http://gdata.youtube.com/schemas/2007#video.related" link */
 	related_link = gdata_entry_look_up_link (GDATA_ENTRY (video), "http://gdata.youtube.com/schemas/2007#video.related");
 	if (related_link == NULL) {
 		/* Erroring out is probably the safest thing to do */
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
-						     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-						     _("The video did not have a related videos <link>."));
+		                                     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+		                                     _("The video did not have a related videos <link>."));
 		return;
 	}
 
 	gdata_service_query_async (GDATA_SERVICE (self), gdata_link_get_uri (related_link), query,
-				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
+	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -772,7 +674,6 @@ gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo
 {
 	/* TODO: Async variant */
 	GDataYouTubeVideo *new_entry;
-	GDataCategory *category;
 	GOutputStream *output_stream;
 	GInputStream *input_stream;
 	const gchar *slug, *content_type, *response_body;
@@ -798,11 +699,6 @@ gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo
 		return NULL;
 	}
 
-	/* Add the "video" kind if the entry is missing it. If it already has the kind category, no duplicate is added. */
-	category = gdata_category_new ("http://gdata.youtube.com/schemas/2007#video", "http://schemas.google.com/g/2005#kind", NULL);
-	gdata_entry_add_category (GDATA_ENTRY (video), category);
-	g_object_unref (category);
-
 	file_info = g_file_query_info (video_file, "standard::display-name,standard::content-type", G_FILE_QUERY_INFO_NONE, NULL, error);
 	if (file_info == NULL)
 		return NULL;
@@ -876,3 +772,122 @@ gdata_youtube_service_get_youtube_user (GDataYouTubeService *self)
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
 	return self->priv->youtube_user;
 }
+
+/**
+ * gdata_youtube_service_get_categories:
+ * @self: a #GDataYouTubeService
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Gets a list of the categories currently in use on YouTube. The returned #GDataAPPCategories contains a list of #GDataYouTubeCategory<!-- -->s which
+ * enumerate the current YouTube categories.
+ *
+ * The category labels (#GDataCategory:label) are localised based on the value of #GDataService:locale.
+ *
+ * Return value: (transfer full): a #GDataAPPCategories, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.7.0
+ **/
+GDataAPPCategories *
+gdata_youtube_service_get_categories (GDataYouTubeService *self, GCancellable *cancellable, GError **error)
+{
+	SoupMessage *message;
+	GDataAPPCategories *categories;
+
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* Download the category list. Note that this is (service) locale-dependent. */
+	message = _gdata_service_query (GDATA_SERVICE (self), "http://gdata.youtube.com/schemas/2007/categories.cat", NULL, cancellable, error);
+	if (message == NULL)
+		return NULL;
+
+	g_assert (message->response_body->data != NULL);
+	categories = GDATA_APP_CATEGORIES (_gdata_parsable_new_from_xml (GDATA_TYPE_APP_CATEGORIES, message->response_body->data,
+	                                                                 message->response_body->length,
+	                                                                 GSIZE_TO_POINTER (GDATA_TYPE_YOUTUBE_CATEGORY), error));
+	g_object_unref (message);
+
+	return categories;
+}
+
+static void
+get_categories_thread (GSimpleAsyncResult *result, GDataYouTubeService *service, GCancellable *cancellable)
+{
+	GDataAPPCategories *categories;
+	GError *error = NULL;
+
+	/* Get the categories and return */
+	categories = gdata_youtube_service_get_categories (service, cancellable, &error);
+	if (error != NULL) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+	}
+
+	g_simple_async_result_set_op_res_gpointer (result, categories, (GDestroyNotify) g_object_unref);
+}
+
+/**
+ * gdata_youtube_service_get_categories_async:
+ * @self: a #GDataYouTubeService
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when the request is finished
+ * @user_data: (closure): data to pass to the @callback function
+ *
+ * Gets a list of the categories currently in use on YouTube. @self is reffed when this function is called, so can safely be unreffed after this
+ * function returns.
+ *
+ * For more details, see gdata_youtube_service_get_categories(), which is the synchronous version of this function.
+ *
+ * When the operation is finished, @callback will be called. You can then call gdata_youtube_service_get_categories_finish() to get the results of the
+ * operation.
+ *
+ * Since: 0.7.0
+ **/
+void
+gdata_youtube_service_get_categories_async (GDataYouTubeService *self, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
+{
+	GSimpleAsyncResult *result;
+
+	g_return_if_fail (GDATA_IS_YOUTUBE_SERVICE (self));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
+
+	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_youtube_service_get_categories_async);
+	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) get_categories_thread, G_PRIORITY_DEFAULT, cancellable);
+	g_object_unref (result);
+}
+
+/**
+ * gdata_youtube_service_get_categories_finish:
+ * @self: a #GDataYouTubeService
+ * @async_result: a #GAsyncResult
+ * @error: a #GError, or %NULL
+ *
+ * Finishes an asynchronous request for a list of categories on YouTube, as started with gdata_youtube_service_get_categories_async().
+ *
+ * Return value: (transfer full): a #GDataAPPCategories, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.7.0
+ **/
+GDataAPPCategories *
+gdata_youtube_service_get_categories_finish (GDataYouTubeService *self, GAsyncResult *async_result, GError **error)
+{
+	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
+	GDataAPPCategories *categories;
+
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == gdata_youtube_service_get_categories_async);
+
+	if (g_simple_async_result_propagate_error (result, error) == TRUE)
+		return NULL;
+
+	categories = g_simple_async_result_get_op_res_gpointer (result);
+	if (categories != NULL)
+		return g_object_ref (categories);
+	return NULL;
+}
