@@ -72,8 +72,6 @@ test_authentication (void)
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
-
-	g_object_unref (service);
 }
 
 static void
@@ -230,36 +228,35 @@ static void
 test_insert_simple (gconstpointer service)
 {
 	GDataCalendarEvent *event, *new_event;
-	GDataCategory *category;
 	GDataGDWhere *where;
 	GDataGDWho *who;
 	GDataGDWhen *when;
 	GTimeVal start_time, end_time;
+	gchar *xml;
 	GError *error = NULL;
 
 	event = gdata_calendar_event_new (NULL);
 
 	gdata_entry_set_title (GDATA_ENTRY (event), "Tennis with Beth");
 	gdata_entry_set_content (GDATA_ENTRY (event), "Meet for a quick lesson.");
-	category = gdata_category_new ("http://schemas.google.com/g/2005#event", "http://schemas.google.com/g/2005#kind", NULL);
-	gdata_entry_add_category (GDATA_ENTRY (event), category);
-	g_object_unref (category);
-	gdata_calendar_event_set_transparency (event, "http://schemas.google.com/g/2005#event.opaque");
-	gdata_calendar_event_set_status (event, "http://schemas.google.com/g/2005#event.confirmed");
+	gdata_calendar_event_set_transparency (event, GDATA_GD_EVENT_TRANSPARENCY_OPAQUE);
+	gdata_calendar_event_set_status (event, GDATA_GD_EVENT_STATUS_CONFIRMED);
 	where = gdata_gd_where_new (NULL, "Rolling Lawn Courts", NULL);
 	gdata_calendar_event_add_place (event, where);
 	g_object_unref (where);
-	who = gdata_gd_who_new ("http://schemas.google.com/g/2005#event.organizer", "John Smith‽", "john.smith@example.com");
+	who = gdata_gd_who_new (GDATA_GD_WHO_EVENT_ORGANIZER, "John Smith‽", "john.smith@example.com");
 	gdata_calendar_event_add_person (event, who);
 	g_object_unref (who);
 	g_time_val_from_iso8601 ("2009-04-17T15:00:00.000Z", &start_time);
 	g_time_val_from_iso8601 ("2009-04-17T17:00:00.000Z", &end_time);
-	when = gdata_gd_when_new (&start_time, &end_time, FALSE);
+	when = gdata_gd_when_new (start_time.tv_sec, end_time.tv_sec, FALSE);
 	gdata_calendar_event_add_time (event, when);
 	g_object_unref (when);
 
 	/* Check the XML */
-	gdata_test_assert_xml (event,
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (event));
+	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -279,6 +276,7 @@ test_insert_simple (gconstpointer service)
 			 		"valueString='John Smith\342\200\275'/>"
 			 	"<gd:where valueString='Rolling Lawn Courts'/>"
 			 "</entry>");
+	g_free (xml);
 
 	/* Insert the event */
 	new_event = gdata_calendar_service_insert_event (GDATA_CALENDAR_SERVICE (service), event, NULL, &error);
@@ -293,12 +291,13 @@ test_insert_simple (gconstpointer service)
 }
 
 static void
-test_xml_dates (gconstpointer service)
+test_xml_dates (void)
 {
 	GDataCalendarEvent *event;
 	GList *times, *i;
 	GDataGDWhen *when;
-	GTimeVal time_val;
+	gint64 _time;
+	gchar *xml;
 	GError *error = NULL;
 
 	event = GDATA_CALENDAR_EVENT (gdata_parsable_new_from_xml (GDATA_TYPE_CALENDAR_EVENT,
@@ -324,12 +323,10 @@ test_xml_dates (gconstpointer service)
 	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next != NULL);
 	g_assert (gdata_gd_when_is_date (when) == TRUE);
-	gdata_gd_when_get_start_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 1239926400);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
-	gdata_gd_when_get_end_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 0);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	_time = gdata_gd_when_get_start_time (when);
+	g_assert_cmpint (_time, ==, 1239926400);
+	_time = gdata_gd_when_get_end_time (when);
+	g_assert_cmpint (_time, ==, -1);
 	g_assert (gdata_gd_when_get_value_string (when) == NULL);
 	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
@@ -338,12 +335,10 @@ test_xml_dates (gconstpointer service)
 	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next != NULL);
 	g_assert (gdata_gd_when_is_date (when) == FALSE);
-	gdata_gd_when_get_start_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 1239926400 + 54000);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
-	gdata_gd_when_get_end_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 0);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	_time = gdata_gd_when_get_start_time (when);
+	g_assert_cmpint (_time, ==, 1239926400 + 54000);
+	_time = gdata_gd_when_get_end_time (when);
+	g_assert_cmpint (_time, ==, -1);
 	g_assert (gdata_gd_when_get_value_string (when) == NULL);
 	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
@@ -352,17 +347,17 @@ test_xml_dates (gconstpointer service)
 	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next == NULL);
 	g_assert (gdata_gd_when_is_date (when) == TRUE);
-	gdata_gd_when_get_start_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 1239926400 + 864000);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
-	gdata_gd_when_get_end_time (when, &time_val);
-	g_assert_cmpint (time_val.tv_sec, ==, 1241568000);
-	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	_time = gdata_gd_when_get_start_time (when);
+	g_assert_cmpint (_time, ==, 1239926400 + 864000);
+	_time = gdata_gd_when_get_end_time (when);
+	g_assert_cmpint (_time, ==, 1241568000);
 	g_assert (gdata_gd_when_get_value_string (when) == NULL);
 	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
 	/* Check the XML */
-	gdata_test_assert_xml (event,
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (event));
+	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -378,12 +373,13 @@ test_xml_dates (gconstpointer service)
 			 	"<gd:when startTime='2009-04-17T15:00:00Z'/>"
 			 	"<gd:when startTime='2009-04-27' endTime='2009-05-06'/>"
 			 "</entry>");
+	g_free (xml);
 
 	g_object_unref (event);
 }
 
 static void
-test_xml_recurrence (gconstpointer service)
+test_xml_recurrence (void)
 {
 	GDataCalendarEvent *event;
 	GError *error = NULL;
@@ -441,15 +437,14 @@ test_xml_recurrence (gconstpointer service)
 	g_assert_cmpstr (id, ==, "g5928e82rrch95b25f8ud0dlsg");
 	g_assert_cmpstr (uri, ==, "http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg");
 
-	g_free (id);
-	g_free (uri);
 	g_object_unref (event);
 }
 
 static void
 test_query_uri (void)
 {
-	GTimeVal time_val, time_val2;
+	gint64 _time;
+	GTimeVal time_val;
 	gchar *query_uri;
 	GDataCalendarQuery *query = gdata_calendar_query_new ("q");
 
@@ -460,16 +455,14 @@ test_query_uri (void)
 	g_assert_cmpstr (gdata_calendar_query_get_order_by (query), ==, "starttime");
 
 	g_time_val_from_iso8601 ("2009-04-17T15:00:00.000Z", &time_val);
-	gdata_calendar_query_set_recurrence_expansion_start (query, &time_val);
-	gdata_calendar_query_get_recurrence_expansion_start (query, &time_val2);
-	g_assert_cmpint (time_val.tv_sec, ==, time_val2.tv_sec);
-	g_assert_cmpint (time_val.tv_usec, ==, time_val2.tv_usec);
+	gdata_calendar_query_set_recurrence_expansion_start (query, time_val.tv_sec);
+	_time = gdata_calendar_query_get_recurrence_expansion_start (query);
+	g_assert_cmpint (_time, ==, time_val.tv_sec);
 
 	g_time_val_from_iso8601 ("2010-04-17T15:00:00.000Z", &time_val);
-	gdata_calendar_query_set_recurrence_expansion_end (query, &time_val);
-	gdata_calendar_query_get_recurrence_expansion_end (query, &time_val2);
-	g_assert_cmpint (time_val.tv_sec, ==, time_val2.tv_sec);
-	g_assert_cmpint (time_val.tv_usec, ==, time_val2.tv_usec);
+	gdata_calendar_query_set_recurrence_expansion_end (query, time_val.tv_sec);
+	_time = gdata_calendar_query_get_recurrence_expansion_end (query);
+	g_assert_cmpint (_time, ==, time_val.tv_sec);
 
 	gdata_calendar_query_set_single_events (query, TRUE);
 	g_assert (gdata_calendar_query_get_single_events (query) == TRUE);
@@ -478,16 +471,14 @@ test_query_uri (void)
 	g_assert_cmpstr (gdata_calendar_query_get_sort_order (query), ==, "descending");
 
 	g_time_val_from_iso8601 ("2009-04-17T15:00:00.000Z", &time_val);
-	gdata_calendar_query_set_start_min (query, &time_val);
-	gdata_calendar_query_get_start_min (query, &time_val2);
-	g_assert_cmpint (time_val.tv_sec, ==, time_val2.tv_sec);
-	g_assert_cmpint (time_val.tv_usec, ==, time_val2.tv_usec);
+	gdata_calendar_query_set_start_min (query, time_val.tv_sec);
+	_time = gdata_calendar_query_get_start_min (query);
+	g_assert_cmpint (_time, ==, time_val.tv_sec);
 
 	g_time_val_from_iso8601 ("2010-04-17T15:00:00.000Z", &time_val);
-	gdata_calendar_query_set_start_max (query, &time_val);
-	gdata_calendar_query_get_start_max (query, &time_val2);
-	g_assert_cmpint (time_val.tv_sec, ==, time_val2.tv_sec);
-	g_assert_cmpint (time_val.tv_usec, ==, time_val2.tv_usec);
+	gdata_calendar_query_set_start_max (query, time_val.tv_sec);
+	_time = gdata_calendar_query_get_start_max (query);
+	g_assert_cmpint (_time, ==, time_val.tv_sec);
 
 	gdata_calendar_query_set_timezone (query, "America/Los Angeles");
 	g_assert_cmpstr (gdata_calendar_query_get_timezone (query), ==, "America/Los_Angeles");
@@ -532,12 +523,12 @@ test_query_etag (void)
 
 	CHECK_ETAG (gdata_calendar_query_set_future_events (query, FALSE))
 	CHECK_ETAG (gdata_calendar_query_set_order_by (query, "shizzle"))
-	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_start (query, NULL))
-	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_end (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_start (query, -1))
+	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_end (query, -1))
 	CHECK_ETAG (gdata_calendar_query_set_single_events (query, FALSE))
 	CHECK_ETAG (gdata_calendar_query_set_sort_order (query, "shizzle"))
-	CHECK_ETAG (gdata_calendar_query_set_start_min (query, NULL))
-	CHECK_ETAG (gdata_calendar_query_set_start_max (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_start_min (query, -1))
+	CHECK_ETAG (gdata_calendar_query_set_start_max (query, -1))
 	CHECK_ETAG (gdata_calendar_query_set_timezone (query, "about now"))
 
 #undef CHECK_ETAG
@@ -573,23 +564,28 @@ test_acls_insert_rule (gconstpointer service)
 	GDataAccessRule *rule, *new_rule;
 	const gchar *scope_type, *scope_value;
 	GDataCategory *category;
+	GDataLink *_link;
 	GList *categories;
+	gchar *xml;
+	gint64 edited;
 	GError *error = NULL;
 
 	calendar = get_calendar (service, &error);
 
 	rule = gdata_access_rule_new (NULL);
 
-	gdata_access_rule_set_role (rule, "http://schemas.google.com/gCal/2005#editor");
-	g_assert_cmpstr (gdata_access_rule_get_role (rule), ==, "http://schemas.google.com/gCal/2005#editor");
+	gdata_access_rule_set_role (rule, GDATA_CALENDAR_ACCESS_ROLE_EDITOR);
+	g_assert_cmpstr (gdata_access_rule_get_role (rule), ==, GDATA_CALENDAR_ACCESS_ROLE_EDITOR);
 
-	gdata_access_rule_set_scope (rule, "user", "darcy@gmail.com");
+	gdata_access_rule_set_scope (rule, GDATA_ACCESS_SCOPE_USER, "darcy@gmail.com");
 	gdata_access_rule_get_scope (rule, &scope_type, &scope_value);
-	g_assert_cmpstr (scope_type, ==, "user");
+	g_assert_cmpstr (scope_type, ==, GDATA_ACCESS_SCOPE_USER);
 	g_assert_cmpstr (scope_value, ==, "darcy@gmail.com");
 
 	/* Check the XML */
-	gdata_test_assert_xml (rule,
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (rule));
+	g_assert_cmpstr (xml, ==,
+			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gAcl='http://schemas.google.com/acl/2007'>"
@@ -598,18 +594,25 @@ test_acls_insert_rule (gconstpointer service)
 				"<gAcl:role value='http://schemas.google.com/gCal/2005#editor'/>"
 				"<gAcl:scope type='user' value='darcy@gmail.com'/>"
 			 "</entry>");
+	g_free (xml);
 
 	/* Insert the rule */
-	new_rule = gdata_access_handler_insert_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
+	_link = gdata_entry_look_up_link (GDATA_ENTRY (calendar), GDATA_LINK_ACCESS_CONTROL_LIST);
+	g_assert (_link != NULL);
+
+	new_rule = GDATA_ACCESS_RULE (gdata_service_insert_entry (GDATA_SERVICE (service), gdata_link_get_uri (_link), GDATA_ENTRY (rule),
+	                                                          NULL, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ACCESS_RULE (new_rule));
 	g_clear_error (&error);
 
 	/* Check the properties of the returned rule */
-	g_assert_cmpstr (gdata_access_rule_get_role (new_rule), ==, "http://schemas.google.com/gCal/2005#editor");
+	g_assert_cmpstr (gdata_access_rule_get_role (new_rule), ==, GDATA_CALENDAR_ACCESS_ROLE_EDITOR);
 	gdata_access_rule_get_scope (new_rule, &scope_type, &scope_value);
-	g_assert_cmpstr (scope_type, ==, "user");
+	g_assert_cmpstr (scope_type, ==, GDATA_ACCESS_SCOPE_USER);
 	g_assert_cmpstr (scope_value, ==, "darcy@gmail.com");
+	edited = gdata_access_rule_get_edited (new_rule);
+	g_assert_cmpuint (edited, >, 0);
 
 	/* Check it only has the one category and that it's correct */
 	categories = gdata_entry_get_categories (GDATA_ENTRY (new_rule));
@@ -624,7 +627,6 @@ test_acls_insert_rule (gconstpointer service)
 
 	g_object_unref (rule);
 	g_object_unref (new_rule);
-	g_object_unref (calendar);
 }
 
 static void
@@ -635,6 +637,7 @@ test_acls_update_rule (gconstpointer service)
 	GDataAccessRule *rule = NULL, *new_rule;
 	const gchar *scope_type, *scope_value;
 	GList *rules;
+	gint64 edited;
 	GError *error = NULL;
 
 	calendar = get_calendar (service, &error);
@@ -659,21 +662,23 @@ test_acls_update_rule (gconstpointer service)
 	g_object_unref (feed);
 
 	/* Update the rule */
-	gdata_access_rule_set_role (rule, "http://schemas.google.com/gCal/2005#read");
-	g_assert_cmpstr (gdata_access_rule_get_role (rule), ==, "http://schemas.google.com/gCal/2005#read");
+	gdata_access_rule_set_role (rule, GDATA_CALENDAR_ACCESS_ROLE_READ);
+	g_assert_cmpstr (gdata_access_rule_get_role (rule), ==, GDATA_CALENDAR_ACCESS_ROLE_READ);
 
 	/* Send the update to the server */
-	new_rule = gdata_access_handler_update_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
+	new_rule = GDATA_ACCESS_RULE (gdata_service_update_entry (GDATA_SERVICE (service), GDATA_ENTRY (rule), NULL, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ACCESS_RULE (new_rule));
 	g_clear_error (&error);
 	g_object_unref (rule);
 
 	/* Check the properties of the returned rule */
-	g_assert_cmpstr (gdata_access_rule_get_role (new_rule), ==, "http://schemas.google.com/gCal/2005#read");
+	g_assert_cmpstr (gdata_access_rule_get_role (new_rule), ==, GDATA_CALENDAR_ACCESS_ROLE_READ);
 	gdata_access_rule_get_scope (new_rule, &scope_type, &scope_value);
-	g_assert_cmpstr (scope_type, ==, "user");
+	g_assert_cmpstr (scope_type, ==, GDATA_ACCESS_SCOPE_USER);
 	g_assert_cmpstr (scope_value, ==, "darcy@gmail.com");
+	edited = gdata_access_rule_get_edited (new_rule);
+	g_assert_cmpuint (edited, >, 0);
 
 	g_object_unref (new_rule);
 	g_object_unref (calendar);
@@ -713,54 +718,271 @@ test_acls_delete_rule (gconstpointer service)
 	g_object_unref (feed);
 
 	/* Delete the rule */
-	success = gdata_access_handler_delete_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
+	success = gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (rule), NULL, &error);
 	g_assert_no_error (error);
 	g_assert (success == TRUE);
 	g_clear_error (&error);
 
 	g_object_unref (rule);
-	g_object_unref (calendar);
+}
+
+static void
+test_batch (gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	GDataService *service2;
+	GDataCalendarCalendar *calendar;
+	GDataCalendarEvent *event, *event2, *event3;
+	GDataEntry *inserted_entry, *inserted_entry2, *inserted_entry3;
+	gchar *feed_uri;
+	guint op_id, op_id2, op_id3;
+	GError *error = NULL, *entry_error = NULL;
+
+	calendar = get_calendar (service, &error);
+
+	/* Here we hardcode the feed URI, but it should really be extracted from an event feed, as the GDATA_LINK_BATCH link */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+
+	/* Check the properties of the operation */
+	g_assert (gdata_batch_operation_get_service (operation) == service);
+	g_assert_cmpstr (gdata_batch_operation_get_feed_uri (operation), ==, "https://www.google.com/calendar/feeds/default/private/full/batch");
+
+	g_object_get (operation,
+	              "service", &service2,
+	              "feed-uri", &feed_uri,
+	              NULL);
+
+	g_assert (service2 == service);
+	g_assert_cmpstr (feed_uri, ==, "https://www.google.com/calendar/feeds/default/private/full/batch");
+
+	g_object_unref (service2);
+	g_free (feed_uri);
+
+	/* Run a singleton batch operation to insert a new entry */
+	event = gdata_calendar_event_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (event), "Fooish Bar");
+
+	gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (event), &inserted_entry, NULL);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (event);
+
+	/* Run another batch operation to insert another entry and query the previous one */
+	event2 = gdata_calendar_event_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (event2), "Cow Lunch");
+
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	op_id = gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (event2), &inserted_entry2, NULL);
+	op_id2 = gdata_test_batch_operation_query (operation, gdata_entry_get_id (inserted_entry), GDATA_TYPE_CALENDAR_EVENT, inserted_entry, NULL,
+	                                           NULL);
+	g_assert_cmpuint (op_id, !=, op_id2);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (event2);
+
+	/* Run another batch operation to delete the first entry and a fictitious one to test error handling, and update the second entry */
+	gdata_entry_set_title (inserted_entry2, "Toby");
+	event3 = gdata_calendar_event_new ("foobar");
+
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	op_id = gdata_test_batch_operation_deletion (operation, inserted_entry, NULL);
+	op_id2 = gdata_test_batch_operation_deletion (operation, GDATA_ENTRY (event3), &entry_error);
+	op_id3 = gdata_test_batch_operation_update (operation, inserted_entry2, &inserted_entry3, NULL);
+	g_assert_cmpuint (op_id, !=, op_id2);
+	g_assert_cmpuint (op_id, !=, op_id3);
+	g_assert_cmpuint (op_id2, !=, op_id3);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR);
+
+	g_clear_error (&error);
+	g_clear_error (&entry_error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry);
+	g_object_unref (event3);
+
+	/* Run another batch operation to update the second entry with the wrong ETag (i.e. pass the old version of the entry to the batch operation
+	 * to test error handling */
+	/* Turns out that we can't run this test, because the Calendar service sucks with ETags and doesn't error. */
+	/*operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	gdata_test_batch_operation_update (operation, inserted_entry2, NULL, &entry_error);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT);
+
+	g_clear_error (&error);
+	g_clear_error (&entry_error);
+	g_object_unref (operation);*/
+	g_object_unref (inserted_entry2);
+
+	/* Run a final batch operation to delete the second entry */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	gdata_test_batch_operation_deletion (operation, inserted_entry3, NULL);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry3);
+}
+
+typedef struct {
+	GDataCalendarEvent *new_event;
+} BatchAsyncData;
+
+static void
+setup_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GDataCalendarEvent *event;
+	GError *error = NULL;
+
+	/* Insert a new event which we can query asyncly */
+	event = gdata_calendar_event_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (event), "Party 'Til You Puke");
+
+	data->new_event = gdata_calendar_service_insert_event (GDATA_CALENDAR_SERVICE (service), event, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_CALENDAR_EVENT (data->new_event));
+	g_clear_error (&error);
+
+	g_object_unref (event);
+}
+
+static void
+test_batch_async_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == TRUE);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+
+	/* Run an async query operation on the event */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_event)), GDATA_TYPE_CALENDAR_EVENT,
+	                                          GDATA_ENTRY (data->new_event), NULL, NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_batch_operation_run_async (operation, NULL, (GAsyncReadyCallback) test_batch_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+}
+
+static void
+test_batch_async_cancellation_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == FALSE);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+	GCancellable *cancellable;
+
+	/* Run an async query operation on the event */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://www.google.com/calendar/feeds/default/private/full/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_event)), GDATA_TYPE_CALENDAR_EVENT,
+	                                          GDATA_ENTRY (data->new_event), NULL, NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+	cancellable = g_cancellable_new ();
+
+	gdata_batch_operation_run_async (operation, cancellable, (GAsyncReadyCallback) test_batch_async_cancellation_cb, main_loop);
+	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+	g_object_unref (cancellable);
+}
+
+static void
+teardown_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GError *error = NULL;
+
+	/* Delete the event */
+	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (data->new_event), NULL, &error) == TRUE);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	g_object_unref (data->new_event);
 }
 
 int
 main (int argc, char *argv[])
 {
 	gint retval;
-	GDataService *service;
+	GDataService *service = NULL;
 
-	g_type_init ();
-	g_thread_init (NULL);
-	g_test_init (&argc, &argv, NULL);
-	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
+	gdata_test_init (argc, argv);
 
-	service = GDATA_SERVICE (gdata_calendar_service_new (CLIENT_ID));
-	gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, NULL);
+	if (gdata_test_internet () == TRUE) {
+		service = GDATA_SERVICE (gdata_calendar_service_new (CLIENT_ID));
+		gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, NULL);
 
-	g_test_add_func ("/calendar/authentication", test_authentication);
-	if (g_test_thorough () == TRUE)
+		g_test_add_func ("/calendar/authentication", test_authentication);
 		g_test_add_func ("/calendar/authentication_async", test_authentication_async);
-	g_test_add_data_func ("/calendar/query/all_calendars", service, test_query_all_calendars);
-	if (g_test_thorough () == TRUE)
+
+		g_test_add_data_func ("/calendar/query/all_calendars", service, test_query_all_calendars);
 		g_test_add_data_func ("/calendar/query/all_calendars_async", service, test_query_all_calendars_async);
-	g_test_add_data_func ("/calendar/query/own_calendars", service, test_query_own_calendars);
-	if (g_test_thorough () == TRUE)
+		g_test_add_data_func ("/calendar/query/own_calendars", service, test_query_own_calendars);
 		g_test_add_data_func ("/calendar/query/own_calendars_async", service, test_query_own_calendars_async);
-	g_test_add_data_func ("/calendar/query/events", service, test_query_events);
-	if (g_test_slow () == TRUE)
+		g_test_add_data_func ("/calendar/query/events", service, test_query_events);
+
 		g_test_add_data_func ("/calendar/insert/simple", service, test_insert_simple);
-	g_test_add_data_func ("/calendar/xml/dates", service, test_xml_dates);
-	g_test_add_data_func ("/calendar/xml/recurrence", service, test_xml_recurrence);
-	g_test_add_func ("/calendar/query/uri", test_query_uri);
-	g_test_add_func ("/calendar/query/etag", test_query_etag);
-	g_test_add_data_func ("/calendar/acls/get_rules", service, test_acls_get_rules);
-	if (g_test_slow () == TRUE) {
+
+		g_test_add_data_func ("/calendar/acls/get_rules", service, test_acls_get_rules);
 		g_test_add_data_func ("/calendar/acls/insert_rule", service, test_acls_insert_rule);
 		g_test_add_data_func ("/calendar/acls/update_rule", service, test_acls_update_rule);
 		g_test_add_data_func ("/calendar/acls/delete_rule", service, test_acls_delete_rule);
+
+		g_test_add_data_func ("/calendar/batch", service, test_batch);
+		g_test_add ("/calendar/batch/async", BatchAsyncData, service, setup_batch_async, test_batch_async, teardown_batch_async);
+		g_test_add ("/calendar/batch/async/cancellation", BatchAsyncData, service, setup_batch_async, test_batch_async_cancellation,
+		            teardown_batch_async);
 	}
 
+	g_test_add_func ("/calendar/xml/dates", test_xml_dates);
+	g_test_add_func ("/calendar/xml/recurrence", test_xml_recurrence);
+
+	g_test_add_func ("/calendar/query/uri", test_query_uri);
+	g_test_add_func ("/calendar/query/etag", test_query_etag);
+
 	retval = g_test_run ();
-	g_object_unref (service);
+
+	if (service != NULL)
+		g_object_unref (service);
 
 	return retval;
 }
