@@ -48,8 +48,6 @@ test_authentication (void)
 	g_assert (gdata_service_is_authenticated (GDATA_SERVICE (service)) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (GDATA_SERVICE (service)), ==, DOCUMENTS_USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (GDATA_SERVICE (service)), ==, PASSWORD);
-
-	g_object_unref (service);
 }
 
 static void
@@ -92,7 +90,6 @@ test_remove_all_documents_and_folders (gconstpointer service)
 		g_clear_error (&error);
 	}
 
-	g_object_unref (query);
 	g_object_unref (feed);
 }
 
@@ -114,7 +111,6 @@ test_query_all_documents_with_folder (gconstpointer service)
 
 	g_clear_error (&error);
 	g_object_unref (feed);
-	g_object_unref (query);
 }
 
 static void
@@ -169,16 +165,12 @@ static void
 test_upload_metadata (gconstpointer service)
 {
 	GDataDocumentsEntry *document, *new_document;
-	GDataCategory *category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_spreadsheet_new (NULL));
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#spreadsheet", "http://schemas.google.com/g/2005#kind", "spreadsheet");
-
 	gdata_entry_set_title (GDATA_ENTRY (document), "myNewSpreadsheet");
-	gdata_entry_add_category (GDATA_ENTRY (document), category);
 
 	/* Insert the document */
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (document), NULL, NULL, NULL, &error);
@@ -195,7 +187,6 @@ test_upload_metadata_file (gconstpointer service)
 {
 	GDataDocumentsEntry *document, *new_document;
 	GFile *document_file;
-	GDataCategory *category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
@@ -203,9 +194,7 @@ test_upload_metadata_file (gconstpointer service)
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.odt");
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#document", "http://schemas.google.com/g/2005#kind", "document");
 	gdata_entry_set_title (GDATA_ENTRY (document), "upload_metadata_file");
-	gdata_entry_add_category (GDATA_ENTRY (document), category);
 
 	/* Insert the document */
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), document, document_file, NULL, NULL, &error);
@@ -221,15 +210,14 @@ test_upload_metadata_file (gconstpointer service)
 static void
 test_upload_file_get_entry (gconstpointer service)
 {
-	GDataDocumentsEntry *new_document, *newly_created_presentation;
+	GDataDocumentsEntry *new_document;
+	GDataEntry *new_presentation;
 	GFile *document_file;
-	GDataCategory *category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
 
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.ppt");
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#presentation", "http://schemas.google.com/g/2005#kind", "presentation");
 
 	/* Insert the document */
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), NULL, document_file, NULL, NULL, &error);
@@ -237,14 +225,14 @@ test_upload_file_get_entry (gconstpointer service)
 	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (new_document));
 
 	/* Get the entry on the server */
-	newly_created_presentation = gdata_documents_service_query_single_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_TYPE_DOCUMENTS_PRESENTATION,
-										    gdata_documents_entry_get_document_id (new_document), NULL, &error);
+	new_presentation = gdata_service_query_single_entry (GDATA_SERVICE (service), gdata_entry_get_id (GDATA_ENTRY (new_document)), NULL,
+	                                                     GDATA_TYPE_DOCUMENTS_PRESENTATION, NULL, &error);
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (newly_created_presentation));
+	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (new_presentation));
 
 	g_clear_error (&error);
 	g_object_unref (new_document);
-	g_object_unref (newly_created_presentation);
+	g_object_unref (new_presentation);
 	g_object_unref (document_file);
 }
 
@@ -254,21 +242,16 @@ test_add_remove_file_from_folder (gconstpointer service)
 	GDataDocumentsEntry *document, *new_document, *new_document2;
 	GDataDocumentsFolder *folder, *new_folder;
 	GFile *document_file;
-	GDataCategory *folder_category, *document_category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
 
 	folder = gdata_documents_folder_new (NULL);
-	folder_category = gdata_category_new ("http://schemas.google.com/docs/2007#folder", "http://schemas.google.com/g/2005#kind", "folder");
 	gdata_entry_set_title (GDATA_ENTRY (folder), "add_remove_from_folder_folder");
-	gdata_entry_add_category (GDATA_ENTRY (folder), folder_category);
 
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.ppt");
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_presentation_new (NULL));
-	document_category = gdata_category_new ("http://schemas.google.com/docs/2007#presentation", "http://schemas.google.com/g/2005#kind", "presentation");
 	gdata_entry_set_title (GDATA_ENTRY (document), "add_remove_from_folder_presentation");
-	gdata_entry_add_category (GDATA_ENTRY (document), document_category);
 
 	/* Insert the folder */
 	new_folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (folder), NULL, NULL, NULL, &error));
@@ -300,7 +283,6 @@ test_add_file_folder_and_move (gconstpointer service)
 	GDataDocumentsEntry *document, *new_document, *new_document2;
 	GDataDocumentsFolder *folder, *new_folder;
 	GFile *document_file;
-	GDataCategory *folder_category, *document_category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
@@ -308,14 +290,10 @@ test_add_file_folder_and_move (gconstpointer service)
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.odt");
 
 	folder = gdata_documents_folder_new (NULL);
-	folder_category = gdata_category_new ("http://schemas.google.com/docs/2007#folder", "http://schemas.google.com/g/2005#kind", "folder");
 	gdata_entry_set_title (GDATA_ENTRY (folder), "add_file_folder_move_folder");
-	gdata_entry_add_category (GDATA_ENTRY (folder), folder_category);
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
-	document_category = gdata_category_new ("http://schemas.google.com/docs/2007#document", "http://schemas.google.com/g/2005#kind", "document");
 	gdata_entry_set_title (GDATA_ENTRY (document), "add_file_folder_move_text");
-	gdata_entry_add_category (GDATA_ENTRY (document), document_category);
 
 	/* Insert the folder */
 	new_folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (folder), NULL, NULL, NULL, &error));
@@ -346,7 +324,6 @@ test_upload_file_metadata_in_new_folder (gconstpointer service)
 	GDataDocumentsEntry *document, *new_document;
 	GDataDocumentsFolder *folder, *new_folder;
 	GFile *document_file;
-	GDataCategory *folder_category, *document_category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
@@ -354,14 +331,10 @@ test_upload_file_metadata_in_new_folder (gconstpointer service)
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.odt");
 
 	folder = gdata_documents_folder_new (NULL);
-	folder_category = gdata_category_new ("http://schemas.google.com/docs/2007#folder", "http://schemas.google.com/g/2005#kind", "folder");
 	gdata_entry_set_title (GDATA_ENTRY (folder), "upload_file_metadata_in_new_folder_folder");
-	gdata_entry_add_category (GDATA_ENTRY (folder), folder_category);
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
-	document_category = gdata_category_new ("http://schemas.google.com/docs/2007#document", "http://schemas.google.com/g/2005#kind", "document");
 	gdata_entry_set_title (GDATA_ENTRY (document), "upload_file_metadata_in_new_folder_text");
-	gdata_entry_add_category (GDATA_ENTRY (document), document_category);
 
 	/* Insert the folder */
 	new_folder = GDATA_DOCUMENTS_FOLDER (gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (folder), NULL, NULL, NULL, &error));
@@ -386,15 +359,12 @@ static void
 test_update_metadata (gconstpointer service)
 {
 	GDataDocumentsEntry *document, *new_document, *updated_document;
-	GDataCategory *category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#document", "http://schemas.google.com/g/2005#kind", "document");
 	gdata_entry_set_title (GDATA_ENTRY (document), "update_metadata_first_title");
-	gdata_entry_add_category (GDATA_ENTRY (document), category);
 
 	/* Insert the document */
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), document, NULL, NULL, NULL, &error);
@@ -420,7 +390,6 @@ test_update_metadata_file (gconstpointer service)
 {
 	GDataDocumentsEntry *document, *new_document, *updated_document;
 	GFile *document_file, *updated_document_file;
-	GDataCategory *category;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
@@ -429,9 +398,7 @@ test_update_metadata_file (gconstpointer service)
 	updated_document_file = g_file_new_for_path (TEST_FILE_DIR "test_updated.odt");
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_text_new (NULL));
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#document", "http://schemas.google.com/g/2005#kind", "document");
 	gdata_entry_set_title (GDATA_ENTRY (document), "update_metadata_file_first_title");
-	gdata_entry_add_category (GDATA_ENTRY (document), category);
 
 	/* Insert the documents metadata*/
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), document, NULL, NULL, NULL, &error);
@@ -501,17 +468,16 @@ test_download_all_documents (gconstpointer service)
 	for (i = gdata_feed_get_entries (GDATA_FEED (feed)); i != NULL; i = i->next) {
 		if (GDATA_IS_DOCUMENTS_PRESENTATION (i->data)) {
 			destination_file = g_file_new_for_path ("/tmp");
-			destination_file = gdata_documents_presentation_download_document (GDATA_DOCUMENTS_PRESENTATION (i->data), GDATA_DOCUMENTS_SERVICE (service),
-											   &content_type, GDATA_DOCUMENTS_PRESENTATION_PPT, destination_file,
-											   TRUE, NULL, &error);
-
+			destination_file = gdata_documents_document_download (GDATA_DOCUMENTS_DOCUMENT (i->data), GDATA_DOCUMENTS_SERVICE (service),
+			                                                      &content_type, GDATA_DOCUMENTS_PRESENTATION_PPT, destination_file,
+			                                                      TRUE, NULL, &error);
 		} else if (GDATA_IS_DOCUMENTS_SPREADSHEET (i->data)) {
 			destination_file_name = g_strdup_printf ("/tmp/%s.%s", gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (i->data)), "ods");
 			destination_file = g_file_new_for_path (destination_file_name);
 			g_free (destination_file_name);
-			destination_file = gdata_documents_spreadsheet_download_document (i->data, GDATA_DOCUMENTS_SERVICE (service),
-											  &content_type, GDATA_DOCUMENTS_SPREADSHEET_ODS, -1, destination_file,
-											  TRUE, NULL, &error);
+			destination_file = gdata_documents_document_download (GDATA_DOCUMENTS_DOCUMENT (i->data), GDATA_DOCUMENTS_SERVICE (service),
+			                                                      &content_type, GDATA_DOCUMENTS_SPREADSHEET_ODS, destination_file,
+			                                                      TRUE, NULL, &error);
 			g_assert_no_error (error);
 
 			destination_display_name = g_string_new (gdata_entry_get_title (GDATA_ENTRY(i->data)));
@@ -529,8 +495,9 @@ test_download_all_documents (gconstpointer service)
 			destination_file_name = g_strdup_printf ("/tmp/%s.%s", gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (i->data)), "odt");
 			destination_file = g_file_new_for_path (destination_file_name);
 			g_free (destination_file_name);
-			destination_file = gdata_documents_text_download_document (i->data, GDATA_DOCUMENTS_SERVICE (service), &content_type, GDATA_DOCUMENTS_TEXT_ODT,
-										   destination_file, TRUE, NULL, &error);
+			destination_file = gdata_documents_document_download (GDATA_DOCUMENTS_DOCUMENT (i->data), GDATA_DOCUMENTS_SERVICE (service),
+			                                                      &content_type, GDATA_DOCUMENTS_TEXT_ODT, destination_file, TRUE, NULL,
+			                                                      &error);
 			g_assert_no_error (error);
 
 			destination_display_name = g_string_new (gdata_entry_get_title (GDATA_ENTRY(i->data)));
@@ -558,16 +525,13 @@ test_new_document_with_collaborator (gconstpointer service)
 {
 	GDataDocumentsEntry *document, *new_document;
 	GDataAccessRule *access_rule, *new_access_rule;
-	GDataCategory *category;
+	GDataLink *_link;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
 
 	document = GDATA_DOCUMENTS_ENTRY (gdata_documents_spreadsheet_new (NULL));
-	category = gdata_category_new ("http://schemas.google.com/docs/2007#spreadsheet", "http://schemas.google.com/g/2005#kind", "spreadsheet");
-
 	gdata_entry_set_title (GDATA_ENTRY (document), "new_with_collaborator");
-	gdata_entry_add_category (GDATA_ENTRY (document), category);
 
 	/* Insert the document */
 	new_document = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (document), NULL, NULL, NULL, &error);
@@ -576,11 +540,15 @@ test_new_document_with_collaborator (gconstpointer service)
 
 	/* New access rule */
 	access_rule = gdata_access_rule_new (NULL);
-	gdata_access_rule_set_role (access_rule, "writer");
-	gdata_access_rule_set_scope (access_rule, "user", "libgdata.test@gmail.com");
+	gdata_access_rule_set_role (access_rule, GDATA_DOCUMENTS_ACCESS_ROLE_WRITER);
+	gdata_access_rule_set_scope (access_rule, GDATA_ACCESS_SCOPE_USER, "libgdata.test@gmail.com");
 
 	/* Set access rules */
-	new_access_rule = gdata_access_handler_insert_rule (GDATA_ACCESS_HANDLER (new_document), GDATA_SERVICE (service), access_rule, NULL, &error);
+	_link = gdata_entry_look_up_link (GDATA_ENTRY (new_document), GDATA_LINK_ACCESS_CONTROL_LIST);
+	g_assert (_link != NULL);
+
+	new_access_rule = GDATA_ACCESS_RULE (gdata_service_insert_entry (GDATA_SERVICE (service), gdata_link_get_uri (_link),
+	                                                                 GDATA_ENTRY (access_rule), NULL, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ACCESS_RULE (new_access_rule));
 
@@ -617,49 +585,300 @@ test_query_etag (void)
 	g_object_unref (query);
 }
 
+static void
+test_batch (gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	GDataService *service2;
+	GDataDocumentsText *doc, *doc2, *doc3;
+	GDataEntry *inserted_entry, *inserted_entry_updated, *inserted_entry2, *inserted_entry2_updated, *inserted_entry3;
+	gchar *feed_uri;
+	guint op_id, op_id2, op_id3;
+	GError *error = NULL, *entry_error = NULL;
+
+	/* Here we hardcode the feed URI, but it should really be extracted from a document feed, as the GDATA_LINK_BATCH link */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+
+	/* Check the properties of the operation */
+	g_assert (gdata_batch_operation_get_service (operation) == service);
+	g_assert_cmpstr (gdata_batch_operation_get_feed_uri (operation), ==, "https://docs.google.com/feeds/documents/private/full/batch");
+
+	g_object_get (operation,
+	              "service", &service2,
+	              "feed-uri", &feed_uri,
+	              NULL);
+
+	g_assert (service2 == service);
+	g_assert_cmpstr (feed_uri, ==, "https://docs.google.com/feeds/documents/private/full/batch");
+
+	g_object_unref (service2);
+	g_free (feed_uri);
+
+	/* Run a singleton batch operation to insert a new entry */
+	doc = gdata_documents_text_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (doc), "My First Document");
+
+	gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (doc), &inserted_entry, NULL);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (doc);
+
+	/* Run another batch operation to insert another entry and query the previous one */
+	doc2 = gdata_documents_text_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (doc2), "I'm a poet and I didn't know it");
+
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	op_id = gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (doc2), &inserted_entry2, NULL);
+	op_id2 = gdata_test_batch_operation_query (operation, gdata_entry_get_id (inserted_entry), GDATA_TYPE_DOCUMENTS_TEXT, inserted_entry, NULL,
+	                                           NULL);
+	g_assert_cmpuint (op_id, !=, op_id2);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (doc2);
+
+	/* Run another batch operation to query one of the entries we just created, since it seems that the ETags for documents change for no
+	 * apparent reason when you're not looking. */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (inserted_entry), GDATA_TYPE_DOCUMENTS_TEXT, inserted_entry,
+	                                  &inserted_entry_updated, NULL);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry);
+
+	/* Run another batch operation to query the other entry we just created. It would be sensible to batch this query together with the previous
+	 * one, seeing as we're testing _batch_ functionality. Funnily enough, the combination of two idempotent operations changes the ETags and
+	 * makes the whole effort worthless. */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (inserted_entry2), GDATA_TYPE_DOCUMENTS_TEXT, inserted_entry2,
+	                                  &inserted_entry2_updated, NULL);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+
+	/* Run another batch operation to delete the first entry and a fictitious one to test error handling, and update the second entry */
+	gdata_entry_set_title (inserted_entry2_updated, "War & Peace");
+	doc3 = gdata_documents_text_new ("foobar");
+
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	op_id = gdata_test_batch_operation_deletion (operation, inserted_entry_updated, NULL);
+	op_id2 = gdata_test_batch_operation_deletion (operation, GDATA_ENTRY (doc3), &entry_error);
+	op_id3 = gdata_test_batch_operation_update (operation, inserted_entry2_updated, &inserted_entry3, NULL);
+	g_assert_cmpuint (op_id, !=, op_id2);
+	g_assert_cmpuint (op_id, !=, op_id3);
+	g_assert_cmpuint (op_id2, !=, op_id3);
+
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR);
+
+	g_clear_error (&error);
+	g_clear_error (&entry_error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry_updated);
+	g_object_unref (inserted_entry2_updated);
+	g_object_unref (doc3);
+
+	/* Run another batch operation to update the second entry with the wrong ETag (i.e. pass the old version of the entry to the batch operation
+	 * to test error handling */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	gdata_test_batch_operation_update (operation, inserted_entry2, NULL, &entry_error);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT);
+
+	g_clear_error (&error);
+	g_clear_error (&entry_error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry2);
+
+	/* Run a final batch operation to delete the second entry */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	gdata_test_batch_operation_deletion (operation, inserted_entry3, NULL);
+	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+
+	g_clear_error (&error);
+	g_object_unref (operation);
+	g_object_unref (inserted_entry3);
+}
+
+typedef struct {
+	GDataDocumentsEntry *new_doc;
+} BatchAsyncData;
+
+static void
+setup_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GDataDocumentsText *doc;
+	GError *error = NULL;
+
+	/* Insert a new document which we can query asyncly */
+	doc = gdata_documents_text_new (NULL);
+	gdata_entry_set_title (GDATA_ENTRY (doc), "A View from the Bridge");
+
+	data->new_doc = gdata_documents_service_upload_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_ENTRY (doc), NULL, NULL, NULL,
+	                                                         &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_DOCUMENTS_TEXT (data->new_doc));
+	g_clear_error (&error);
+
+	g_object_unref (doc);
+}
+
+static void
+test_batch_async_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == TRUE);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+
+	/* Run an async query operation on the document */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_doc)), GDATA_TYPE_DOCUMENTS_TEXT,
+	                                          GDATA_ENTRY (data->new_doc), NULL, NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	gdata_batch_operation_run_async (operation, NULL, (GAsyncReadyCallback) test_batch_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+}
+
+static void
+test_batch_async_cancellation_cb (GDataBatchOperation *operation, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	GError *error = NULL;
+
+	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == FALSE);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
+{
+	GDataBatchOperation *operation;
+	guint op_id;
+	GMainLoop *main_loop;
+	GCancellable *cancellable;
+
+	/* Run an async query operation on the document */
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "https://docs.google.com/feeds/documents/private/full/batch");
+	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_doc)), GDATA_TYPE_DOCUMENTS_TEXT,
+	                                          GDATA_ENTRY (data->new_doc), NULL, NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+	cancellable = g_cancellable_new ();
+
+	gdata_batch_operation_run_async (operation, cancellable, (GAsyncReadyCallback) test_batch_async_cancellation_cb, main_loop);
+	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
+
+	g_main_loop_run (main_loop);
+	g_main_loop_unref (main_loop);
+	g_object_unref (cancellable);
+}
+
+static void
+teardown_batch_async (BatchAsyncData *data, gconstpointer service)
+{
+	GDataEntry *document;
+	GError *error = NULL;
+
+	/* Re-query the document in case its ETag has changed */
+	document = gdata_service_query_single_entry (GDATA_SERVICE (service), gdata_entry_get_id (GDATA_ENTRY (data->new_doc)), NULL,
+	                                             GDATA_TYPE_DOCUMENTS_TEXT, NULL, &error);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	/* Delete the document */
+	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), document, NULL, &error) == TRUE);
+	g_assert_no_error (error);
+	g_clear_error (&error);
+
+	g_object_unref (data->new_doc);
+	g_object_unref (document);
+}
+
 int
 main (int argc, char *argv[])
 {
-	GDataService *service;
 	gint retval;
+	GDataService *service = NULL;
 
-	g_type_init ();
-	g_thread_init (NULL);
-	g_test_init (&argc, &argv, NULL);
-	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
+	gdata_test_init (argc, argv);
 
-	service = GDATA_SERVICE (gdata_documents_service_new (CLIENT_ID));
-	gdata_service_authenticate (service, DOCUMENTS_USERNAME, PASSWORD, NULL, NULL);
+	if (gdata_test_internet () == TRUE) {
+		service = GDATA_SERVICE (gdata_documents_service_new (CLIENT_ID));
+		gdata_service_authenticate (service, DOCUMENTS_USERNAME, PASSWORD, NULL, NULL);
 
-	g_test_add_func ("/documents/authentication", test_authentication);
+		g_test_add_func ("/documents/authentication", test_authentication);
 
-	g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);
+		g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);
 
-	g_test_add_data_func ("/documents/upload/only_file_get_entry", service, test_upload_file_get_entry);
-	g_test_add_data_func ("/documents/upload/metadata_file", service, test_upload_metadata_file);
-	g_test_add_data_func ("/documents/upload/only_metadata", service, test_upload_metadata);
-	g_test_add_data_func ("/documents/upload/metadata_file_in_new_folder", service, test_upload_file_metadata_in_new_folder);
-	g_test_add_data_func ("/documents/download/download_all_documents", service, test_download_all_documents);
-	g_test_add_data_func ("/documents/update/only_metadata", service, test_update_metadata);
-	g_test_add_data_func ("/documents/update/only_file", service, test_update_file);
-	g_test_add_data_func ("/documents/update/metadata_file", service, test_update_metadata_file);
+		g_test_add_data_func ("/documents/upload/only_file_get_entry", service, test_upload_file_get_entry);
+		g_test_add_data_func ("/documents/upload/metadata_file", service, test_upload_metadata_file);
+		g_test_add_data_func ("/documents/upload/only_metadata", service, test_upload_metadata);
+		g_test_add_data_func ("/documents/upload/metadata_file_in_new_folder", service, test_upload_file_metadata_in_new_folder);
 
-	g_test_add_data_func ("/documents/access_rules/add_document_with_a_collaborator", service, test_new_document_with_collaborator);
+		g_test_add_data_func ("/documents/download/download_all_documents", service, test_download_all_documents);
 
-	g_test_add_data_func ("/documents/query/all_documents_with_folder", service, test_query_all_documents_with_folder);
-	g_test_add_data_func ("/documents/query/all_documents", service, test_query_all_documents);
-	if (g_test_thorough () == TRUE)
+		g_test_add_data_func ("/documents/update/only_metadata", service, test_update_metadata);
+		g_test_add_data_func ("/documents/update/only_file", service, test_update_file);
+		g_test_add_data_func ("/documents/update/metadata_file", service, test_update_metadata_file);
+
+		g_test_add_data_func ("/documents/access_rules/add_document_with_a_collaborator", service, test_new_document_with_collaborator);
+
+		g_test_add_data_func ("/documents/query/all_documents_with_folder", service, test_query_all_documents_with_folder);
+		g_test_add_data_func ("/documents/query/all_documents", service, test_query_all_documents);
 		g_test_add_data_func ("/documents/query/all_documents_async", service, test_query_all_documents_async);
 
-	g_test_add_data_func ("/documents/move/move_to_folder", service, test_add_file_folder_and_move);
-	g_test_add_data_func ("/documents/move/remove_from_folder", service, test_add_remove_file_from_folder);
-	/*g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);*/
+		g_test_add_data_func ("/documents/move/move_to_folder", service, test_add_file_folder_and_move);
+		g_test_add_data_func ("/documents/move/remove_from_folder", service, test_add_remove_file_from_folder);
+		/*g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);*/
+
+		g_test_add_data_func ("/documents/batch", service, test_batch);
+		g_test_add ("/documents/batch/async", BatchAsyncData, service, setup_batch_async, test_batch_async, teardown_batch_async);
+		g_test_add ("/documents/batch/async/cancellation", BatchAsyncData, service, setup_batch_async, test_batch_async_cancellation,
+		            teardown_batch_async);
+	}
 
 	g_test_add_func ("/documents/query/etag", test_query_etag);
 
 	retval = g_test_run ();
 
-	g_object_unref (service);
+	if (service != NULL)
+		g_object_unref (service);
 
 	return retval;
 }
