@@ -24,7 +24,7 @@
 #include <config.h>
 
 /* For the thumbnail size tests in test_download_thumbnails() */
-#ifdef HAVE_GDK
+#ifdef HAVE_GDK_PIXBUF
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
@@ -62,7 +62,6 @@ delete_directory (GFile *directory, GError **error)
 	}
 
 	g_file_enumerator_close (enumerator, NULL, error);
-	g_object_unref (enumerator);
 
 	/* Delete the directory itself */
 	g_file_delete (directory, NULL, error);
@@ -165,30 +164,33 @@ test_upload_async (gconstpointer service)
 	GRegex *regex;
 	GMatchInfo *match_info;
 	guint64 delta;
+	const gchar * const tags[] = { "foo", "bar", ",,baz,baz", NULL };
 	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
-
 	g_get_current_time (&timeval);
 	time_str = g_time_val_to_iso8601 (&timeval);
 	summary = g_strdup_printf ("Async Photo Summary (%s)", time_str);
 
-	expected_xml = g_strdup_printf ("<entry ("
-						"xmlns='http://www.w3.org/2005/Atom' ?|"
-						"xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
-						"xmlns:media='http://search.yahoo.com/mrss/' ?|"
-						"xmlns:gd='http://schemas.google.com/g/2005' ?|"
-						"xmlns:exif='http://schemas.google.com/photos/exif/2007' ?|"
-						"xmlns:app='http://www.w3.org/2007/app' ?|"
-						"xmlns:georss='http://www.georss.org/georss' ?|"
-						"xmlns:gml='http://www.opengis.net/gml' ?){8}>"
+	expected_xml = g_strdup_printf ("<entry "
+						"xmlns='http://www.w3.org/2005/Atom' "
+						"xmlns:gphoto='http://schemas.google.com/photos/2007' "
+						"xmlns:media='http://search.yahoo.com/mrss/' "
+						"xmlns:gd='http://schemas.google.com/g/2005' "
+						"xmlns:exif='http://schemas.google.com/photos/exif/2007' "
+						"xmlns:app='http://www.w3.org/2007/app' "
+						"xmlns:georss='http://www.georss.org/georss' "
+						"xmlns:gml='http://www.opengis.net/gml'>"
 						"<title type='text'>Async Photo Entry Title</title>"
 						"<summary type='text'>Async Photo Summary \\(%s\\)</summary>"
+						"<category term='http://schemas.google.com/photos/2007#photo' "
+							"scheme='http://schemas.google.com/g/2005#kind'/>"
 						"<gphoto:position>0</gphoto:position>"
 						"<gphoto:timestamp>([0-9]+)</gphoto:timestamp>"
 						"<gphoto:commentingEnabled>true</gphoto:commentingEnabled>"
 						"<media:group>"
 							"<media:title type='plain'>Async Photo Entry Title</media:title>"
 							"<media:description type='plain'>Async Photo Summary \\(%s\\)</media:description>"
+							"<media:keywords>foo,bar,%%2C%%2Cbaz%%2Cbaz</media:keywords>"
 						"</media:group>"
 					"</entry>", time_str, time_str);
 	g_free (time_str);
@@ -201,15 +203,16 @@ test_upload_async (gconstpointer service)
 	photo = gdata_picasaweb_file_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (photo), "Async Photo Entry Title");
 	gdata_picasaweb_file_set_caption (photo, summary);
+	gdata_picasaweb_file_set_tags (photo, tags);
 
 	/* Check the XML: match it against the regex built above, then check that the timestamp is within 100ms of the current time at the start of
 	 * the test function. We can't check it exactly, as a few milliseconds may have passed inbetween building the expected_xml and building the XML
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 2);
+	parsed_time_str = g_match_info_fetch (match_info, 1);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
-	g_assert_cmpuint (abs (delta), <, 100);
+	g_assert_cmpuint (abs (delta), <, 1000);
 
 	g_free (parsed_time_str);
 	g_free (xml);
@@ -357,16 +360,16 @@ test_download_thumbnails (gconstpointer _service)
 
 	/* test getting all thumbnails and that they're all the correct size */
 	for (node = thumbnails; node != NULL; node = node->next) {
-#ifdef HAVE_GDK
+#ifdef HAVE_GDK_PIXBUF
 		GdkPixbuf *pixbuf;
-#endif /* HAVE_GDK */
+#endif /* HAVE_GDK_PIXBUF */
 
 		thumbnail = GDATA_MEDIA_THUMBNAIL (node->data);
 		actual_file = gdata_media_thumbnail_download (thumbnail, service, "thumbnail.jpg", dest_file, FALSE, NULL, &error);
 		g_assert_no_error (error);
 		g_assert (g_file_query_exists (actual_file, NULL));
 
-#ifdef HAVE_GDK
+#ifdef HAVE_GDK_PIXBUF
 		file_path = g_file_get_path (actual_file);
 		pixbuf = gdk_pixbuf_new_from_file (file_path, &error);
 		g_assert_no_error (error);
@@ -376,7 +379,7 @@ test_download_thumbnails (gconstpointer _service)
 		g_assert_cmpint (abs (gdk_pixbuf_get_width (pixbuf) - (gint)gdata_media_thumbnail_get_width (thumbnail)) , <=, 1);
 		g_assert_cmpint (abs (gdk_pixbuf_get_height (pixbuf) - (gint)gdata_media_thumbnail_get_height (thumbnail)) , <=, 1);
 		g_object_unref (pixbuf);
-#endif /* HAVE_GDK */
+#endif /* HAVE_GDK_PIXBUF */
 
 		g_file_delete (actual_file, NULL, &error);
 		g_assert (g_file_query_exists (actual_file, NULL) == FALSE);
@@ -535,28 +538,33 @@ test_upload_simple (gconstpointer service)
 	GRegex *regex;
 	GMatchInfo *match_info;
 	guint64 delta;
+	const gchar * const tags[] = { "foo", "bar", ",,baz,baz", NULL };
+	const gchar * const *tags2;
 
 	g_get_current_time (&timeval);
 	time_str = g_time_val_to_iso8601 (&timeval);
 	summary = g_strdup_printf ("Photo Summary (%s)", time_str);
 
-	expected_xml = g_strdup_printf ("<entry ("
-						"xmlns='http://www.w3.org/2005/Atom' ?|"
-						"xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
-						"xmlns:media='http://search.yahoo.com/mrss/' ?|"
-						"xmlns:gd='http://schemas.google.com/g/2005' ?|"
-						"xmlns:exif='http://schemas.google.com/photos/exif/2007' ?|"
-						"xmlns:app='http://www.w3.org/2007/app' ?|"
-						"xmlns:georss='http://www.georss.org/georss' ?|"
-						"xmlns:gml='http://www.opengis.net/gml' ?){8}>"
+	expected_xml = g_strdup_printf ("<entry "
+						"xmlns='http://www.w3.org/2005/Atom' "
+						"xmlns:gphoto='http://schemas.google.com/photos/2007' "
+						"xmlns:media='http://search.yahoo.com/mrss/' "
+						"xmlns:gd='http://schemas.google.com/g/2005' "
+						"xmlns:exif='http://schemas.google.com/photos/exif/2007' "
+						"xmlns:app='http://www.w3.org/2007/app' "
+						"xmlns:georss='http://www.georss.org/georss' "
+						"xmlns:gml='http://www.opengis.net/gml'>"
 						"<title type='text'>Photo Entry Title</title>"
 						"<summary type='text'>Photo Summary \\(%s\\)</summary>"
+						"<category term='http://schemas.google.com/photos/2007#photo' "
+							"scheme='http://schemas.google.com/g/2005#kind'/>"
 						"<gphoto:position>0</gphoto:position>"
 						"<gphoto:timestamp>([0-9]+)</gphoto:timestamp>"
 						"<gphoto:commentingEnabled>true</gphoto:commentingEnabled>"
 						"<media:group>"
 							"<media:title type='plain'>Photo Entry Title</media:title>"
 							"<media:description type='plain'>Photo Summary \\(%s\\)</media:description>"
+							"<media:keywords>foo,bar,%%2C%%2Cbaz%%2Cbaz</media:keywords>"
 						"</media:group>"
 					"</entry>", time_str, time_str);
 	g_free (time_str);
@@ -569,15 +577,16 @@ test_upload_simple (gconstpointer service)
 	photo = gdata_picasaweb_file_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (photo), "Photo Entry Title");
 	gdata_picasaweb_file_set_caption (photo, summary);
+	gdata_picasaweb_file_set_tags (photo, tags);
 
 	/* Check the XML: match it against the regex built above, then check that the timestamp is within 100ms of the current time at the start of
 	 * the test function. We can't check it exactly, as a few milliseconds may have passed inbetween building the expected_xml and building the XML
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 2);
+	parsed_time_str = g_match_info_fetch (match_info, 1);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
-	g_assert_cmpuint (abs (delta), <, 100);
+	g_assert_cmpuint (abs (delta), <, 1000);
 
 	g_free (parsed_time_str);
 	g_free (xml);
@@ -597,6 +606,11 @@ test_upload_simple (gconstpointer service)
 	g_clear_error (&error);
 
 	/* TODO: check entries and feed properties */
+	tags2 = gdata_picasaweb_file_get_tags (photo_new);
+	g_assert_cmpuint (g_strv_length ((gchar**) tags2), ==, 3);
+	g_assert_cmpstr (tags2[0], ==, tags[0]);
+	g_assert_cmpstr (tags2[1], ==, tags[1]);
+	g_assert_cmpstr (tags2[2], ==, tags[2]);
 
 	g_free (summary);
 	g_object_unref (photo);
@@ -619,9 +633,7 @@ test_photo (gconstpointer service)
 	GList *list;
 	GDataMediaContent *content;
 	GDataMediaThumbnail *thumbnail;
-	GTimeVal _time;
-	gchar *str;
-	gchar *timestamp;
+	const gchar * const *tags;
 	gdouble latitude;
 	gdouble longitude;
 	gdouble original_latitude;
@@ -646,10 +658,7 @@ test_photo (gconstpointer service)
 	photo_entry = GDATA_ENTRY (g_list_nth_data (files, 0));
 	photo = GDATA_PICASAWEB_FILE (photo_entry);
 
-	gdata_picasaweb_file_get_edited (photo, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2010-05-13T20:45:13.603000Z");
-	g_free (str);
+	g_assert_cmpint (gdata_picasaweb_file_get_edited (photo), ==, 1273783513);
 
 	/* tests */
 
@@ -662,11 +671,7 @@ test_photo (gconstpointer service)
 	g_assert_cmpuint (gdata_picasaweb_file_get_size (photo), ==, 1124730);
 	/* TODO: file wasn't uploaded with client assigned; g_assert_cmpstr (gdata_picasaweb_file_get_client (photo), ==, ??); */
 	/* TODO: file wasn't uploaded with checksum assigned; g_assert_cmpstr (gdata_picasaweb_file_get_checksum (photo), ==, ??); */
-
-	gdata_picasaweb_file_get_timestamp (photo, &_time);
-	timestamp = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (timestamp, ==, "2008-12-06T18:32:10Z");
-
+	g_assert_cmpint (gdata_picasaweb_file_get_timestamp (photo), ==, 1228588330000);
 	g_assert_cmpstr (gdata_picasaweb_file_get_video_status (photo), ==, NULL);
 	/* TODO: not a good test of video status; want to upload a video for it */
 	g_assert_cmpuint (gdata_picasaweb_file_is_commenting_enabled (photo), ==, TRUE);
@@ -674,7 +679,10 @@ test_photo (gconstpointer service)
 	g_assert_cmpuint (gdata_picasaweb_file_get_rotation (photo), ==, 0);
 
 	g_assert_cmpstr (gdata_picasaweb_file_get_caption (photo), ==, "Ginger cookie caption");
-	g_assert_cmpstr (gdata_picasaweb_file_get_tags (photo), ==, "cookies");
+	tags = gdata_picasaweb_file_get_tags (photo);
+	g_assert (tags != NULL);
+	g_assert_cmpstr (tags[0], ==, "cookies");
+	g_assert (tags[1] == NULL);
 	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (photo)), ==, "100_0269.jpg");
 
 	g_assert_cmpstr (gdata_picasaweb_file_get_credit (photo), ==, "libgdata.picasaweb");
@@ -737,10 +745,6 @@ test_photo (gconstpointer service)
 	g_assert_cmpuint (gdata_media_thumbnail_get_width (thumbnail), ==, 288);
 	g_assert_cmpuint (gdata_media_thumbnail_get_height (thumbnail), ==, 216);
 	g_assert_cmpint (gdata_media_thumbnail_get_time (thumbnail), ==, -1); /* PicasaWeb doesn't set anything better */
-
-	g_free (timestamp);
-	g_object_unref (album_feed);
-	g_object_unref (photo_feed);
 }
 
 static void
@@ -753,10 +757,7 @@ test_photo_feed_entry (gconstpointer service)
 	GDataPicasaWebAlbum *album;
 	GList *albums;
 	GList *files;
-	gchar *xml;
 	GDataEntry *photo_entry;
-	gchar *str;
-	GTimeVal _time;
 
 	album_feed = gdata_picasaweb_service_query_all_albums (GDATA_PICASAWEB_SERVICE (service), NULL, NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
@@ -784,27 +785,13 @@ test_photo_feed_entry (gconstpointer service)
 	g_assert_cmpstr (gdata_picasaweb_file_get_id (GDATA_PICASAWEB_FILE (photo_entry)), ==, "5328890138794566386");
 	g_assert_cmpstr (gdata_entry_get_id (photo_entry), ==, "http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249/photoid/5328890138794566386");
 	g_assert_cmpstr (gdata_entry_get_etag (photo_entry), !=, NULL);
-
-	gdata_entry_get_updated (photo_entry, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2010-05-13T20:45:13.603000Z");
-	g_free (str);
-
-	gdata_entry_get_published (photo_entry, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2009-04-26T06:55:20Z");
-	g_free (str);
-
-	g_assert_cmpstr (gdata_entry_get_content (photo_entry), ==,
+	g_assert_cmpint (gdata_entry_get_updated (photo_entry), ==, 1273783513);
+	g_assert_cmpint (gdata_entry_get_published (photo_entry), ==, 1240728920);
+	g_assert (gdata_entry_get_content (photo_entry) == NULL);
+	g_assert_cmpstr (gdata_entry_get_content_uri (photo_entry), ==,
 			 "http://lh3.ggpht.com/_1kdcGyvOb8c/SfQFWPnuovI/AAAAAAAAAB0/MI0L4Sd11Eg/100_0269.jpg");
-
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry));
-	g_assert_cmpstr (xml, !=, NULL);
-	g_assert_cmpuint (strlen (xml), >, 0);
-	g_free (xml);
-
-	g_object_unref (album_feed);
-	g_object_unref (photo_feed);
+	g_assert_cmpstr (gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry)), !=, NULL);
+	g_assert_cmpuint (strlen (gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry))), >, 0);
 }
 
 static void
@@ -841,9 +828,27 @@ test_photo_feed (gconstpointer service)
 	g_assert_cmpuint (gdata_feed_get_items_per_page (photo_feed), ==, 1000);
 	g_assert_cmpuint (gdata_feed_get_start_index (photo_feed), ==, 1);
 	g_assert_cmpuint (gdata_feed_get_total_results (photo_feed), ==, 1);
+}
 
-	g_object_unref (album_feed);
-	g_object_unref (photo_feed);
+static void
+test_photo_single (gconstpointer service)
+{
+	GDataEntry *photo;
+	GError *error = NULL;
+
+	photo = gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                          "http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249/photoid/5328890138794566386",
+	                                          NULL, GDATA_TYPE_PICASAWEB_FILE, NULL, &error);
+
+	g_assert_no_error (error);
+	g_assert (photo != NULL);
+	g_assert (GDATA_IS_PICASAWEB_FILE (photo));
+	g_assert_cmpstr (gdata_picasaweb_file_get_id (GDATA_PICASAWEB_FILE (photo)), ==, "5328890138794566386");
+	g_assert_cmpstr (gdata_entry_get_id (photo), ==,
+	                 "http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249/photoid/5328890138794566386");
+	g_clear_error (&error);
+
+	g_object_unref (photo);
 }
 
 static void
@@ -852,10 +857,8 @@ test_album (gconstpointer service)
 	GDataFeed *album_feed;
 	GDataPicasaWebAlbum *album;
 	GList *albums, *contents, *thumbnails;
-	GTimeVal _time;
-	gchar *str, *original_rights;
+	gchar *original_rights;
 	gdouble latitude, longitude, original_latitude, original_longitude;
-	const gchar *tags;
 	GDataMediaContent *content;
 	GDataMediaThumbnail *thumbnail;
 	GError *error = NULL;
@@ -875,20 +878,10 @@ test_album (gconstpointer service)
 	/* Check album-specific API */
 	g_assert_cmpstr (gdata_picasaweb_album_get_user (album), ==, "libgdata.picasaweb");
 	g_assert_cmpstr (gdata_picasaweb_album_get_nickname (album), ==, "libgdata.picasaweb");
-
-	gdata_picasaweb_album_get_edited (album, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2009-04-26T06:57:03.474000Z");
-	g_free (str);
-
+	g_assert_cmpint (gdata_picasaweb_album_get_edited (album), ==, 1240729023);
 	g_assert_cmpint (gdata_picasaweb_album_get_visibility (album), ==, GDATA_PICASAWEB_PUBLIC);
 	g_assert_cmpstr (gdata_picasaweb_album_get_location (album), ==, "Venice");
-
-	gdata_picasaweb_album_get_timestamp (album, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2009-04-26T07:00:00Z");
-	g_free (str);
-
+	g_assert_cmpint (gdata_picasaweb_album_get_timestamp (album), ==, 1240729200000);
 	g_assert_cmpuint (gdata_picasaweb_album_get_num_photos (album), ==, 1);
 	g_assert_cmpuint (gdata_picasaweb_album_get_num_photos_remaining (album), >, 0); /* about 999 remaining, testing weakly to avoid having to update regularly */
 	g_assert_cmpuint (gdata_picasaweb_album_get_bytes_used (album), ==, 1124730);
@@ -938,8 +931,7 @@ test_album (gconstpointer service)
 	g_free (original_rights);
 
 	/* Check Media */
-	tags = gdata_picasaweb_album_get_tags (album);
-	g_assert_cmpstr (gdata_picasaweb_album_get_tags (album), ==, NULL);
+	g_assert (gdata_picasaweb_album_get_tags (album) == NULL);
 	/* TODO: they return a <media:keywords></...> but it's empty and the web interface can't set it;
 	   try setting it programmatically; if we can't do that either, consider removing API */
 
@@ -979,8 +971,7 @@ test_album_feed_entry (gconstpointer service)
 	GError *error = NULL;
 	GDataEntry *entry;
 	GList *albums;
-	gchar *str, *xml;
-	GTimeVal _time;
+	gchar *xml;
 
 	album_feed = gdata_picasaweb_service_query_all_albums (GDATA_PICASAWEB_SERVICE (service), NULL, NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
@@ -1002,16 +993,8 @@ test_album_feed_entry (gconstpointer service)
 	g_assert_cmpstr (gdata_entry_get_id (entry), ==, "http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249");
 	g_assert_cmpstr (gdata_entry_get_etag (entry), !=, NULL);
 	g_assert_cmpstr (gdata_entry_get_rights (entry), ==, "public");
-
-	gdata_entry_get_updated (entry, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2009-04-26T06:57:03.474000Z");
-	g_free (str);
-
-	gdata_entry_get_published (entry, &_time);
-	str = g_time_val_to_iso8601 (&_time);
-	g_assert_cmpstr (str, ==, "2009-04-26T07:00:00Z");
-	g_free (str);
+	g_assert_cmpint (gdata_entry_get_updated (entry), ==, 1240729023);
+	g_assert_cmpint (gdata_entry_get_published (entry), ==, 1240729200);
 
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
 	g_assert_cmpstr (xml, !=, NULL);
@@ -1038,12 +1021,10 @@ test_album_feed (gconstpointer service)
 	/* TODO find out why subtitle == null when returned: no subtitle for feed? printf("feed subtitle: %s\n", gdata_feed_get_subtitle(feed)); */
 	g_assert_cmpstr (gdata_feed_get_id (album_feed), ==, "http://picasaweb.google.com/data/feed/user/libgdata.picasaweb");
 	g_assert_cmpstr (gdata_feed_get_etag (album_feed), !=, NULL); /* this varies as albums change, like when a new image is uploaded in our test! */
-	g_assert_cmpstr (gdata_feed_get_icon (album_feed), ==, "http://lh3.ggpht.com/_1kdcGyvOb8c/AAAAAAAAAAA/AAAAAAAAAAA/jNhBfdaNdD4/s64-c/libgdata.picasaweb.jpg");
+	g_assert_cmpstr (gdata_feed_get_icon (album_feed), ==, "http://lh6.ggpht.com/_1kdcGyvOb8c/AAAA9mDag3s/AAAAAAAAAAA/Jq-NWYWKFao/s64-c/libgdata.picasaweb.jpg");
 	g_assert_cmpuint (gdata_feed_get_items_per_page (album_feed), ==, 1000);
 	g_assert_cmpuint (gdata_feed_get_start_index (album_feed), ==, 1);
 	g_assert_cmpuint (gdata_feed_get_total_results (album_feed), ==, NUM_ALBUMS);
-
-	g_object_unref (album_feed);
 }
 
 static void
@@ -1052,7 +1033,6 @@ test_insert_album (gconstpointer service)
 	GDataPicasaWebAlbum *album;
 	GDataPicasaWebAlbum *inserted_album;
 	GTimeVal timestamp;
-	gchar *timestr;
 	GError *error;
 
 	GDataFeed *album_feed;
@@ -1070,7 +1050,7 @@ test_insert_album (gconstpointer service)
 	gdata_picasaweb_album_set_location (album, "Winnipeg, MN");
 
 	g_time_val_from_iso8601 ("2002-10-14T09:58:59.643554Z", &timestamp);
-	gdata_picasaweb_album_set_timestamp (album, &timestamp);
+	gdata_picasaweb_album_set_timestamp (album, timestamp.tv_sec * 1000);
 
 	inserted_album = gdata_picasaweb_service_insert_album (GDATA_PICASAWEB_SERVICE (service), album, NULL, &error);
 	g_assert_no_error (error);
@@ -1080,11 +1060,7 @@ test_insert_album (gconstpointer service)
 	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (inserted_album)), ==, "Thanksgiving photos");
 	g_assert_cmpstr (gdata_entry_get_summary (GDATA_ENTRY (inserted_album)), ==, "Family photos of the feast!");
 	g_assert_cmpstr (gdata_picasaweb_album_get_location (inserted_album), ==, "Winnipeg, MN");
-
-	gdata_picasaweb_album_get_timestamp (inserted_album, &timestamp);
-	timestr = g_time_val_to_iso8601 (&timestamp);
-	g_assert_cmpstr (timestr, ==, "2002-10-14T09:58:59Z");
-	g_free (timestr);
+	g_assert_cmpint (gdata_picasaweb_album_get_timestamp (inserted_album), ==, 1034589539000);
 
 	/* Test that album is actually on server */
 	album_feed = gdata_picasaweb_service_query_all_albums (GDATA_PICASAWEB_SERVICE (service), NULL, NULL, NULL, NULL, NULL, &error);
@@ -1124,7 +1100,6 @@ test_query_all_albums (gconstpointer service)
 	g_assert_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER);
 	g_assert (album_feed == NULL);
 	g_clear_error (&error);
-	g_object_unref (query);
 
 	/* Now try a proper query */
 	album_feed = gdata_picasaweb_service_query_all_albums (GDATA_PICASAWEB_SERVICE (service), NULL, NULL, NULL, NULL, NULL, &error);
@@ -1162,7 +1137,7 @@ test_query_user (gconstpointer service)
 	g_assert_cmpint (gdata_picasaweb_user_get_quota_limit (user), ==, 1073741824); /* 1GiB: it'll be a beautiful day when this assert gets tripped */
 	g_assert_cmpint (gdata_picasaweb_user_get_quota_current (user), >, 0);
 	g_assert_cmpint (gdata_picasaweb_user_get_max_photos_per_album (user), >, 0); /* now it's 1000, testing this weakly to avoid having to regularly update it */
-	g_assert_cmpstr (gdata_picasaweb_user_get_thumbnail_uri (user), ==, "http://lh3.ggpht.com/_1kdcGyvOb8c/AAAAAAAAAAA/AAAAAAAAAAA/jNhBfdaNdD4/s64-c/libgdata.picasaweb.jpg");
+	g_assert_cmpstr (gdata_picasaweb_user_get_thumbnail_uri (user), ==, "http://lh6.ggpht.com/_1kdcGyvOb8c/AAAA9mDag3s/AAAAAAAAAAA/Jq-NWYWKFao/s64-c/libgdata.picasaweb.jpg");
 
 	g_object_unref (user);
 }
@@ -1252,7 +1227,7 @@ test_query_all_albums_async (gconstpointer service)
 }
 
 static void
-test_album_new (gconstpointer service)
+test_album_new (void)
 {
 	GDataPicasaWebAlbum *album;
 	gchar *xml, *parsed_time_str;
@@ -1266,17 +1241,18 @@ test_album_new (gconstpointer service)
 	/* Get the current time */
 	g_get_current_time (&timeval);
 
-	/* Build a regex to match the timestamp from the XML, since we can't definitely say what it'll be. Note that we also assign any order to the
-	 * namespace definitions, since due to a change in GLib's hashing algorithm, they could be in different orders with different GLib versions. */
-	regex = g_regex_new ("<entry (xmlns='http://www.w3.org/2005/Atom' ?|"
-				     "xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
-				     "xmlns:media='http://search.yahoo.com/mrss/' ?|"
-				     "xmlns:gd='http://schemas.google.com/g/2005' ?|"
-				     "xmlns:gml='http://www.opengis.net/gml' ?|"
-				     "xmlns:app='http://www.w3.org/2007/app' ?|"
-				     "xmlns:georss='http://www.georss.org/georss' ?){7}>"
+	/* Build a regex to match the timestamp from the XML, since we can't definitely say what it'll be */
+	regex = g_regex_new ("<entry xmlns='http://www.w3.org/2005/Atom' "
+				    "xmlns:gphoto='http://schemas.google.com/photos/2007' "
+				    "xmlns:media='http://search.yahoo.com/mrss/' "
+				    "xmlns:gd='http://schemas.google.com/g/2005' "
+				    "xmlns:gml='http://www.opengis.net/gml' "
+				    "xmlns:app='http://www.w3.org/2007/app' "
+				    "xmlns:georss='http://www.georss.org/georss'>"
 					"<title type='text'></title>"
 					"<id>http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249</id>"
+					"<category term='http://schemas.google.com/photos/2007#album' "
+						"scheme='http://schemas.google.com/g/2005#kind'/>"
 					"<gphoto:id>5328889949261497249</gphoto:id>"
 					"<gphoto:access>private</gphoto:access>"
 					"<gphoto:timestamp>([0-9]+)</gphoto:timestamp>"
@@ -1293,9 +1269,9 @@ test_album_new (gconstpointer service)
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (album));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 2);
+	parsed_time_str = g_match_info_fetch (match_info, 1);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
-	g_assert_cmpuint (abs (delta), <, 100);
+	g_assert_cmpuint (abs (delta), <, 1000);
 
 	g_free (parsed_time_str);
 	g_free (xml);
@@ -1335,43 +1311,49 @@ test_query_etag (void)
 int
 main (int argc, char *argv[])
 {
-	GDataService *service;
 	gint retval;
+	GDataService *service = NULL;
 
-	g_type_init ();
-	g_thread_init (NULL);
-	g_test_init (&argc, &argv, NULL);
-	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
+	gdata_test_init (argc, argv);
 
-	service = GDATA_SERVICE (gdata_picasaweb_service_new (CLIENT_ID));
-	gdata_service_authenticate (service, PW_USERNAME, PASSWORD, NULL, NULL);
+	if (gdata_test_internet () == TRUE) {
+		service = GDATA_SERVICE (gdata_picasaweb_service_new (CLIENT_ID));
+		gdata_service_authenticate (service, PW_USERNAME, PASSWORD, NULL, NULL);
 
-	g_test_add_func ("/picasaweb/authentication", test_authentication);
-	if (g_test_thorough () == TRUE)
+		g_test_add_func ("/picasaweb/authentication", test_authentication);
 		g_test_add_func ("/picasaweb/authentication_async", test_authentication_async);
-	g_test_add_data_func ("/picasaweb/upload/photo", service, test_upload_simple);
-	if (g_test_thorough () == TRUE)
+
+		g_test_add_data_func ("/picasaweb/upload/photo", service, test_upload_simple);
 		g_test_add_data_func ("/picasaweb/upload/photo_async", service, test_upload_async);
-	g_test_add_data_func ("/picasaweb/query/all_albums", service, test_query_all_albums);
-	g_test_add_data_func ("/picasaweb/query/user", service, test_query_user);
-	if (g_test_thorough () == TRUE)
+
+		g_test_add_data_func ("/picasaweb/query/all_albums", service, test_query_all_albums);
+		g_test_add_data_func ("/picasaweb/query/user", service, test_query_user);
 		g_test_add_data_func ("/picasaweb/query/all_albums_async", service, test_query_all_albums_async);
-	g_test_add_data_func ("/picasaweb/query/new_with_limits", service, test_query_new_with_limits);
-	g_test_add_data_func ("/picasaweb/query/album_feed", service, test_album_feed);
-	g_test_add_data_func ("/picasaweb/query/album_feed_entry", service, test_album_feed_entry);
-	g_test_add_data_func ("/picasaweb/query/album", service, test_album);
-	g_test_add_data_func ("/picasaweb/insert/album", service, test_insert_album);
-	g_test_add_data_func ("/picasaweb/query/photo_feed", service, test_photo_feed);
-	g_test_add_data_func ("/picasaweb/query/photo_feed_entry", service, test_photo_feed_entry);
-	g_test_add_data_func ("/picasaweb/query/photo", service, test_photo);
-	g_test_add_data_func ("/picasaweb/upload/photo", service, test_upload_simple);
-	g_test_add_data_func ("/picasaweb/download/photo", service, test_download);
-	g_test_add_data_func ("/picasaweb/download/thumbnails", service, test_download_thumbnails);
-	g_test_add_data_func ("/picasaweb/album/new", service, test_album_new);
+		g_test_add_data_func ("/picasaweb/query/new_with_limits", service, test_query_new_with_limits);
+		g_test_add_data_func ("/picasaweb/query/album_feed", service, test_album_feed);
+		g_test_add_data_func ("/picasaweb/query/album_feed_entry", service, test_album_feed_entry);
+		g_test_add_data_func ("/picasaweb/query/album", service, test_album);
+
+		g_test_add_data_func ("/picasaweb/insert/album", service, test_insert_album);
+
+		g_test_add_data_func ("/picasaweb/query/photo_feed", service, test_photo_feed);
+		g_test_add_data_func ("/picasaweb/query/photo_feed_entry", service, test_photo_feed_entry);
+		g_test_add_data_func ("/picasaweb/query/photo", service, test_photo);
+		g_test_add_data_func ("/picasaweb/query/photo_single", service, test_photo_single);
+
+		g_test_add_data_func ("/picasaweb/upload/photo", service, test_upload_simple);
+
+		g_test_add_data_func ("/picasaweb/download/photo", service, test_download);
+		g_test_add_data_func ("/picasaweb/download/thumbnails", service, test_download_thumbnails);
+	}
+
+	g_test_add_func ("/picasaweb/album/new", test_album_new);
 	g_test_add_func ("/picasaweb/query/etag", test_query_etag);
 
 	retval = g_test_run ();
-	g_object_unref (service);
+
+	if (service != NULL)
+		g_object_unref (service);
 
 	return retval;
 }
